@@ -60,6 +60,7 @@ POLL = "5"
 
 # Tokens
 TOKEN_ATTRIBUTE = "A!"
+TOKEN_ATTRIBUTE_REPLACE = "R!"
 TOKEN_ALL = "*"
 
 # Words
@@ -67,6 +68,8 @@ RISING = u"Rising"
 FALLING = u"Falling"
 POSITIVE = u"Positive"
 NEGATIVE = u"Negative"
+ROWS = u"Rows"
+COLUMNS = u"Columns"
 
 # Macro steps
 MACRO_SET_ATTRIBUTE_STEP = u"Set attribute"
@@ -341,3 +344,24 @@ ATTR_MAX_RUNNING_MACROS = u"Max running macros"
 ATTR_MAX_MACRO_HISTORY = u"Max macro history"
 ATTR_TEMPLATE = u"Template"
 ATTR_CHAIN_LINK_OUTPUT = u"Chain link output"
+ATTR_OFF = u"Off"
+ATTR_PS1_VOLTAGE = u"PS 1 voltage"
+ATTR_PS1_POWER = u"PS 1 power"
+ATTR_PS1_CURRENT = u"PS 1 current"
+ATTR_PS2_VOLTAGE = u"PS 2 voltage"
+ATTR_PS2_POWER = u"PS 2 power"
+ATTR_PS2_CURRENT = u"PS 2 current"
+ATTR_PS3_VOLTAGE = u"PS 3 voltage"
+ATTR_PS3_POWER = u"PS 3 power"
+ATTR_PS3_CURRENT = u"PS 3 current"
+ATTR_PS4_VOLTAGE = u"PS 4 voltage"
+ATTR_PS4_POWER = u"PS 4 power"
+ATTR_PS4_CURRENT = u"PS 4 current"
+ATTR_POWER_SETPOINT = u"Power setpoint"
+ATTR_VOLTAGE_SETPOINT = u"Voltage setpoint"
+ATTR_CURRENT_SETPOINT = u"Current setpoint"
+ATTR_OUTPUT_RELEASED = u"Output released"
+ATTR_PROCESS_ID = u"Process ID"
+ATTR_MEMORY_USAGE = u"Memory usage"
+ATTR_CPU_USAGE = u"Cpu usage"
+ATTR_AVAILABLE_MEMORY = u"Available memory"
