@@ -4,18 +4,13 @@ from threading import Lock
 from time import time
 
 import pint
-from numpy import dtype, float16, float32, float64, uint16, uint32, uint64, int16, int32, int64, uint8, int8, ndarray, \
-    bool
-from pint import UndefinedUnitError
+from numpy import dtype, float16, float32, float64, uint16, uint32, uint64, int16, int32, int64, uint8, int8, ndarray
 
 from kamzik3 import units
 from kamzik3.constants import *
 
 
 # TODO: Deal with numpy arrays
-# TODO: Implement option to force attribute to change it's value even if current one is the same
-#       It's needed for some casses like counter
-# TODO: Remove current workarounds introduced for Karabo bridge
 
 class SetFunction(object):
 
@@ -47,25 +42,24 @@ class Attribute(dict):
     def attribute_copy(self):
         return Attribute(**self)
 
-    def __init__(self, default_value=None, min_value=None, max_value=None,
-                 unit=None, set_function=None, default_type=u"str", decimals=0,
-                 readonly=False, description=None, set_value_when_set_function=True,
-                 tolerance=(0, 0), factor=1, offset=0, **kwargs):
+    def __init__(self, attribute_id=None, default_value=None, min_value=None, max_value=None, unit=None,
+                 set_function=None, default_type=u"str", decimals=0, readonly=False, description=None,
+                 set_value_when_set_function=True, tolerance=(0, 0), factor=1, offset=0, **kwargs):
         dict.__init__(self)
-        self.set_function = set_function
         self.update_lock = Lock()
+        self._callbacks = {}
+        self.attribute_id = attribute_id
+        self.set_function = set_function
         self.default_type = default_type
-        self.offset = offset
-        self.factor = factor
+        self[OFFSET] = offset
+        self[FACTOR] = factor
         # Set attribute value flag when using set function.
         # This flag prevents change of attribute's value before value is actually set on device side.
         self.set_value_when_set_function = set_value_when_set_function
-        self._callbacks = {}
         if unit is None:
             unit = u""
         self[UNIT] = unit
-        if kwargs.get(TYPE, False) is not False:
-            default_type = kwargs.get(TYPE)
+        default_type = kwargs.get(TYPE, default_type)
         if default_type == TYPE_LIST:
             self[TYPE] = TYPE_LIST
         elif isinstance(default_type, (list, collections.abc.KeysView)):
@@ -89,7 +83,8 @@ class Attribute(dict):
                 self[MIN] = float(min_value) if min_value is not None else -math.inf
                 self[MAX] = float(max_value) if max_value is not None else math.inf
                 self.numerical = True
-        self[TOLERANCE] = [float(tolerance[0]), float(tolerance[1])]
+        if self.numerical:
+            self[TOLERANCE] = [float(tolerance[0]), float(tolerance[1])]
         self[READONLY] = readonly
         self[DESCRIPTION] = description
         self[VALUE] = default_value
@@ -100,69 +95,82 @@ class Attribute(dict):
         if self.update_lock.locked():
             return
 
-        if k == FACTOR:
-            self.factor = v
-        elif k == OFFSET:
-            self.offset = v
-        elif k == VALUE:
+        if k == VALUE:
             if isinstance(v, units.Quantity):
                 try:
                     v = v.to(self[UNIT]).m
                 except AttributeError:
                     v = v.m
-            if self.set_function and self.with_set_function:
-                self[SETPOINT] = v
-                setpoint = self.remove_offset_factor(v)
-                # Release set function flag
-                self.with_set_function = False
-                # Call set function
-                if hasattr(self.set_function, "callback"):
-                    self.set_token = self.set_function(setpoint, callback=self.set_callback)
-                else:
-                    # We don't have callback in a function, so just call it plain without callback
-                    self.set_token = self.set_function(setpoint)
-                if not self.set_value_when_set_function:
-                    return
-            else:
-                v = self.apply_offset_factor(v)
-        elif k in (MIN, MAX):
-            v = self.apply_offset_factor(v)
         elif k == TOLERANCE:
             # Make sure that tolerance is float
             v = [float(t) for t in v]
         elif k == UNIT and v is not None:
-            if v == "%":
+            if v == u"%":
                 v = u"percent"
             if v not in ("", None):
-                try:
-                    # Try to get unit name from pint
-                    units.get_name(v)
-                except pint.errors.UndefinedUnitError:
-                    # Unit does not exists so it's added to the pint definitions
+                if units.parse_unit_name(v) == ():
                     units.define(u"{}=1".format(v))
                     units.derived_units.append(v)
+                # try:
+                    # Try to get unit name from pint
+                    # print(v, units.parse_unit_name("degree"))
+                    # units.get_name(v)
+                # except pint.errors.UndefinedUnitError:
+                #     # Unit does not exists so it's added to the pint definitions
+                #     units.define(u"{}=1".format(v))
+                #     units.derived_units.append(v)
+        try:
+            if self[k] == v:
+                return
+        except KeyError:
+            pass
 
-        # Store actual value into dictionary
-        super(Attribute, self).__setitem__(k, v)
+        if k == VALUE and self.set_function and self.with_set_function:
+            self[SETPOINT] = v
+            setpoint = self.remove_offset_factor(v)
+            # Release set function flag
+            self.with_set_function = False
+            if self.set_value_when_set_function:
+                # Store actual attribute[k] value into dictionary
+                super(Attribute, self).__setitem__(k, v)
+            # Call set function
+            if hasattr(self.set_function, "callback"):
+                self.set_token = self.set_function(setpoint, callback=self.set_callback)
+            else:
+                # We don't have callback in a function, so just call it plain without callback
+                self.set_token = self.set_function(setpoint)
+            if not self.set_value_when_set_function:
+                return
+        else:
+            # Store actual attribute[k] value into dictionary
+            super(Attribute, self).__setitem__(k, v)
 
-        # Call all relevant callbacks
-        for callback, (max_update_rate, updated_at) in self._callbacks.copy().items():
-            if k != VALUE or max_update_rate is None or time() - updated_at > max_update_rate:
-                callback(k, v)
-                try:
-                    self._callbacks[callback][1] = time()
-                except KeyError:
-                    # Callback no longer exists, skipping
-                    pass
+        self.handle_callbacks(k, v)
 
+    def handle_callbacks(self, k, v):
+        # Set time of last update to now
         self.last_update_time = time()
 
-    def __getitem__(self, k):
-        if k == FACTOR:
-            return self.factor
-        elif k == OFFSET:
-            return self.offset
-        return super(Attribute, self).__getitem__(k)
+        # Call all relevant callbacks
+        for callback, (max_update_rate, updated_at, key_filter) in self._callbacks.copy().items():
+            # Check update rate, continue if it's less than max_update_rate
+            if k == VALUE and max_update_rate is not None and (time() - updated_at) < max_update_rate:
+                continue
+            # Check if key_filter is set
+            if key_filter is None:
+                # Key can be any value, therefore callback with key and value
+                callback(k, v)
+            elif k == key_filter:
+                # Since we have filter set, callback with just value
+                callback(v)
+            else:
+                continue
+            try:
+                # Set last callback time
+                self._callbacks[callback][1] = self.last_update_time
+            except KeyError:
+                # Callback no longer exists, skipping
+                pass
 
     @staticmethod
     def from_dict(dictionary):
@@ -199,7 +207,7 @@ class Attribute(dict):
         except (AttributeError, TypeError, ValueError):
             return False
 
-    def attach_callback(self, callback, max_update_rate=None):
+    def attach_callback(self, callback, max_update_rate=None, key_filter=None):
         """
         Attach callback to Attribute.
         :param callback: callable function
@@ -211,7 +219,7 @@ class Attribute(dict):
         if callback not in self._callbacks:
             if max_update_rate is not None:
                 max_update_rate *= 1e-3
-            self._callbacks[callback] = [max_update_rate, time()]
+            self._callbacks[callback] = [max_update_rate, time(), key_filter]
 
     def detach_callback(self, callback):
         """
@@ -231,10 +239,13 @@ class Attribute(dict):
         :param value: numerical value
         :return: mixed
         """
+        if value is None:
+            value = self[VALUE]
+
         if value is not None and self.numerical:
-            return (value + self.offset) * self.factor
+            return (value + self[OFFSET]) * self[FACTOR]
         elif self.default_type is bool:
-            return self.factor == value
+            return self[FACTOR] == value
         else:
             return value
 
@@ -244,10 +255,13 @@ class Attribute(dict):
         :param value: numerical value
         :return: mixed
         """
+        if value is None:
+            value = self[VALUE]
+
         if value is not None and self.numerical:
-            return (value / self.factor) - self.offset
+            return (value / self[FACTOR]) - self[OFFSET]
         elif self.default_type is bool:
-            return self.factor == value
+            return self[FACTOR] == value
         else:
             return value
 
@@ -299,7 +313,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[VALUE], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[VALUE])
             except TypeError:
                 return self[VALUE]
@@ -310,7 +324,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[MIN], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[MIN])
             except TypeError:
                 return self[VALUE]
@@ -321,7 +335,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[MAX], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[MAX])
             except TypeError:
                 return self[VALUE]
@@ -332,7 +346,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[TOLERANCE][0], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[TOLERANCE][0])
             except TypeError:
                 return self[VALUE]
@@ -343,7 +357,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[TOLERANCE][1], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[TOLERANCE][1])
             except TypeError:
                 return self[VALUE]
@@ -354,7 +368,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[OFFSET], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[OFFSET])
             except TypeError:
                 return self[VALUE]
@@ -365,7 +379,7 @@ class Attribute(dict):
         if self.numerical:
             try:
                 return units.Quantity(self[SETPOINT], self[UNIT])
-            except (UndefinedUnitError, AttributeError):
+            except (pint.UndefinedUnitError, AttributeError):
                 return units.Quantity(self[SETPOINT])
             except TypeError:
                 return self[SETPOINT]
