@@ -1,7 +1,6 @@
 import logging
 import time
 from logging.handlers import TimedRotatingFileHandler
-from threading import Thread, Event
 
 import numpy as np
 
@@ -10,6 +9,7 @@ from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
 from kamzik3.snippets.snippetLogging import set_rotating_file_handler, set_file_handler
+from kamzik3.snippets.snippetsTimer import CallbackTimer
 
 ATTR_LOGFILE = "Logfile"
 ATTR_HEADER = "Header"
@@ -26,16 +26,16 @@ class AttributeLogger(Device):
     attribute_logger = None
     attribute_logger_handler = None
     preset_header = u""
+    logger_ticker = None
 
     def __init__(self, log_file_name, device_id=None, config=None):
         if self.logged_attributes is None:
             self.logged_attributes = []
 
-        self.stopped = Event()
         self.log_file_name = log_file_name
         self.log_formatter = logging.Formatter(self.separator.join(['%(asctime)s', '%(created)s', '%(message)s']),
                                                datefmt='%Y-%m-%d %H:%M:%S')
-        super(AttributeLogger, self).__init__(device_id, config)
+        Device.__init__(self, device_id, config)
         self.connect()
 
     def connect(self):
@@ -44,15 +44,18 @@ class AttributeLogger(Device):
 
     def _init_attributes(self):
         super(AttributeLogger, self)._init_attributes()
-        self.attributes.update({
-            ATTR_HEADER: Attribute(readonly=True),
-            ATTR_INTERVAL: Attribute(self.interval, min_value=100, max_value=3600 * 1e3, unit=u"ms",
-                                     default_type=np.uint32),
-            ATTR_LAST_LOG_LINE: Attribute(readonly=True),
-            ATTR_LOGFILE: Attribute(self.log_file_name, readonly=True),
-            ATTR_LOGGING: Attribute(min_value=0, max_value=1, default_type=np.bool, set_function=self.start,
-                                    set_value_when_set_function=True)
-        })
+        self.create_attribute(ATTR_HEADER, readonly=True)
+        self.create_attribute(ATTR_INTERVAL, default_value=self.interval, min_value=100, max_value=3600 * 1e3,
+                              unit=u"ms", default_type=np.uint32, set_function=self._set_timeout)
+        self.create_attribute(ATTR_LAST_LOG_LINE, readonly=True)
+        self.create_attribute(ATTR_LOGFILE, default_value=self.log_file_name, readonly=True)
+        self.create_attribute(ATTR_LOGGING, min_value=0, max_value=1, default_type=np.bool, set_function=self.start)
+
+    def command(self, command, callback=None, with_token=False, returning=True):
+        raise DeviceError("Device does not accept any commands.")
+
+    def _set_timeout(self, timeout):
+        self.timeout = timeout * 1e-3
 
     def _get_logged_attributes(self):
         logged_values = []
@@ -80,7 +83,7 @@ class AttributeLogger(Device):
         return logged_values
 
     def set_log_file_name(self, log_file_name):
-        self.set_attribute([ATTR_LOGFILE, VALUE], log_file_name)
+        self.set_raw_value(ATTR_LOGFILE, log_file_name)
         attribute_logger = logging.getLogger(u"AttributeLogger.{}".format(self.device_id))
         if self.attribute_logger_handler is not None:
             attribute_logger.removeHandler(self.attribute_logger_handler)
@@ -115,7 +118,7 @@ class AttributeLogger(Device):
     def handle_interval_timeout(self):
         logged_values = self._get_logged_attributes()
         last_log_line = self.separator.join(logged_values)
-        self.set_attribute([ATTR_LAST_LOG_LINE, VALUE], last_log_line)
+        self.set_raw_value(ATTR_LAST_LOG_LINE, last_log_line)
         self.attribute_logger.info(last_log_line)
 
     def add_logged_attribute(self, device_id, attribute):
@@ -141,42 +144,31 @@ class AttributeLogger(Device):
 
     def write_header(self):
         header = self.preset_header + self.generate_header()
-        self.set_attribute([ATTR_HEADER, VALUE], header)
+        self.set_raw_value(ATTR_HEADER, header)
         with open(self.log_file_name, "a+") as fp:
             fp.write(header)
             fp.write("\n")
 
     def start(self, flag=True):
         if flag:
-            if self.is_status(STATUS_BUSY):
+            if self.logger_ticker is not None and self.is_status(STATUS_BUSY):
+                self.logger_ticker.stop()
                 raise DeviceError(u"Logging is already running")
 
-            self.stopped.clear()
+            self.logger_ticker = CallbackTimer(self.interval, self.handle_interval_timeout, with_correction=True)
             self.write_header()
-
-            Thread(target=self._logging_thread).start()
+            self.set_status(STATUS_BUSY)
+            self.logger_ticker.start()
         else:
             self.stop()
 
     def stop(self):
         try:
-            self.stopped.set()
+            self.logger_ticker.stop()
             self.set_status(STATUS_IDLE)
         except TypeError:
             pass
 
-    def _logging_thread(self):
-        self.set_status(STATUS_BUSY)
-        # Set waiting time to zero initially so we always get the first log input
-        logging_interval = 0
-        while not self.stopped.wait(logging_interval):
-            t = time.time()
-            self.handle_interval_timeout()
-            # Time compensation for time needed to collect log input
-            time_delta = time.time() - t
-            # Calculate next logging interval
-            logging_interval = self.get_value(ATTR_INTERVAL) * 1e-3 - time_delta
-
     def close(self):
         self.stop()
         super(AttributeLogger, self).close()
@@ -205,7 +197,6 @@ class AttributeLoggerTriggered(AttributeLogger):
             if self.is_status(STATUS_BUSY):
                 raise DeviceError(u"Logging is already running")
 
-            self.stopped.clear()
             self.write_header()
             self.set_status(STATUS_BUSY)
         else:
@@ -213,3 +204,9 @@ class AttributeLoggerTriggered(AttributeLogger):
 
     def trigger(self):
         self.handle_interval_timeout()
+
+    def stop(self):
+        try:
+            self.set_status(STATUS_IDLE)
+        except TypeError:
+            pass
