@@ -1,3 +1,4 @@
+import inspect
 import logging
 import time
 from collections import deque
@@ -10,12 +11,12 @@ import serial
 from pint import UndefinedUnitError, DimensionalityError
 
 import kamzik3
-from kamzik3 import WriteException, CommandFormatException, DeviceError, units
+from kamzik3 import WriteException, CommandFormatException, DeviceError
 from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute, SetFunction
 from kamzik3.devices.subject import Subject
-from kamzik3.snippets.snippetDataAccess import get_from_dict, set_in_dict, fullname
-from kamzik3.snippets.snippetsControlLoops import control_device_poller
+from kamzik3.snippets.snippetDataAccess import get_from_dict, fullname
+from kamzik3.snippets.snippetsControlLoops import control_device_poller, control_device_connection_poller
 from kamzik3.snippets.snippetsGenerators import device_id_generator, token_generator
 from kamzik3.snippets.snippetsUnits import device_units
 from kamzik3.snippets.snippetsYaml import YamlSerializable
@@ -46,46 +47,46 @@ class Device(Subject, YamlSerializable):
                 : close()
     """
     id_generator = device_id_generator()
-    device_id = None
     connection_timeout = 4000
     response_timeout = 4000
     allow_reconnect = True
-    config = None
-    # Size of command buffer used in push
-    push_buffer_size = 2 ** 16
     device_server = None
     session = None
     # Maximum number of commands joined in one request
-    push_commands_max = inf
     max_command_retry = 0
     command_retry_count = 0
+    push_commands_max = inf
+    push_buffer_size = 2 ** 16
 
     def __init__(self, device_id=None, config=None):
         Subject.__init__(self)
-        self.device_poller = control_device_poller
         self.device_id = device_id
-        self.config = config
-        self.qualified_name = fullname(self)
-        if self.config is None:
-            self.config = {}
         if device_id is None:
             self.device_id = next(Device.id_generator)
+        self.config = config
+        if self.config is None:
+            self.config = {}
+        self.token_generator = token_generator()
+        self.device_poller = control_device_poller
+        self.device_connection_poller = control_device_connection_poller
+        # Get absolute path to the Device class
+        self.qualified_name = fullname(self)
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Device.{}".format(self.device_id))
-        if self.config.get("push_commands_max", None) is not None:
-            self.push_commands_max = self.config.get("push_commands_max")
-        if self.config.get("push_buffer_size", None) is not None:
+        if "push_commands_max" in self.config:
+            self.push_commands_max = self.config["push_commands_max"]
+        # Size of command buffer used in push
+        if "push_buffer_size" in self.config:
             self.push_buffer_size = self.config.get("push_buffer_size")
-        if not self.push_commands_max:
-            self.push_commands_max = inf
         self.macro_steps = {MACRO_SET_ATTRIBUTE_STEP: ["*"], MACRO_EXECUTE_METHOD_STEP: ["*"]}
         self.exposed_methods = []
+        self._expose_methods_to_clients()
         # Check if attributes are already defined
         if not hasattr(self, "attributes"):
             self.attributes = {}
             self.attributes_sharing_map = {}
+            self.attribute_attach_map = {}
             self._init_attributes()
-        self.token_generator = token_generator()
         self.connecting_time = 0
         self.response_timestamp = 0
         self.request_timestamp = 0
@@ -96,8 +97,7 @@ class Device(Subject, YamlSerializable):
         self.connection_error = False
         self.response_error = False
         self.latency_buffer = []
-        self.init_time = time.time()
-        self.init_timeout = self.init_time
+        self.init_timeout = self.init_time = time.time()
         self.commands_buffer = deque()
         if kamzik3.session is not None and self.device_id not in kamzik3.session.devices:
             self.set_session(kamzik3.session)
@@ -119,64 +119,68 @@ class Device(Subject, YamlSerializable):
             if key == VALUE:
                 self.notify(ATTR_STATUS, value)
 
-        status_attribute = Attribute(STATUS_DISCONNECTED, readonly=True, description="Current device status")
-        status_attribute.attach_callback(set_status)
+        self.create_attribute(ATTR_ID, default_value=self.device_id, readonly=True, description="Unique device ID")
+        self.create_attribute(ATTR_STATUS, default_value=STATUS_DISCONNECTED, readonly=True,
+                              description="Current device status").attach_callback(set_status)
+        self.create_attribute(ATTR_DESCRIPTION, readonly=True, description="Device description")
+        self.create_attribute(ATTR_ENABLED, default_value=True, default_type=np.bool,
+                              description="Allow any external changes to device")
+        self.create_attribute(ATTR_LATENCY, default_value=0, default_type=np.uint16,
+                              description="Latency between sending command and receiving an answer",
+                              min_value=0, max_value=9999, unit="ms", readonly=True)
+        self.create_attribute(ATTR_BUFFERED_COMMANDS, default_value=0, default_type=np.uint32,
+                              description="Amount of commands waiting to be executed",
+                              min_value=0, readonly=True)
+        self.create_attribute(ATTR_HANGING_COMMANDS, default_value=0, default_type=np.uint32,
+                              description="Amount of commands waiting to be answered from device",
+                              min_value=0, readonly=True)
+        self.create_attribute(ATTR_LAST_ERROR, readonly=True, description="Last device error exception")
 
-        self.attributes = {
-            ATTR_ID: Attribute(self.device_id, readonly=True, description="Unique device ID"),
-            ATTR_STATUS: status_attribute,
-            ATTR_DESCRIPTION: Attribute(readonly=True, description="Device description"),
-            ATTR_ENABLED: Attribute(True, default_type=np.bool, description="Allow any external changes to device"),
-            ATTR_LATENCY: Attribute(0, default_type=np.uint16,
-                                    description="Latency between sending command and receiving an answer",
-                                    min_value=0, max_value=9999, unit="ms", readonly=True),
-            ATTR_BUFFERED_COMMANDS: Attribute(0, default_type=np.uint32,
-                                              description="Amount of commands waiting to be executed",
-                                              min_value=0, readonly=True),
-            ATTR_HANGING_COMMANDS: Attribute(0, default_type=np.uint32,
-                                             description="Amount of commands waiting to be answered from device",
-                                             min_value=0, readonly=True),
-            ATTR_LAST_ERROR: Attribute(None, readonly=True, description="Last device error exception"),
-        }
+    def create_attribute(self, name, group=None, **kwargs):
+        attribute_path = name
+        if group is not None:
+            attribute_path = "{}.{}".format(group, attribute_path)
+        attribute_id = "{}.{}.{}".format(self.device_id, TOKEN_ATTRIBUTE, attribute_path)
+        attribute = Attribute(attribute_id, **kwargs)
 
-    def share_attribute(self, source_device, source_attribute_group, target_attribute_group, shared_attributes=ALL):
-        """
-        Share attribute ensures that attributes from one device are mirrored to current device.
-        On the client side we use attributes_sharing_map to see which devices, groups and attributes are involved.
-        Using this approach we can save traffic. We don't have to use data twice, just once and simply use subscription
-        to attributes topic.
-        :param source_device: Device
-        :param source_attribute_group: Attribute group we want to share or None
-        :param target_attribute_group: Attribute group where we want to put shared attributes
-        :param shared_attributes: list of attributes we want to share or ALL for all attributes in group
-        :return: None
-        """
-        if target_attribute_group is None:
-            target = self.attributes
+        if group is None:
+            self.attributes[name] = attribute
         else:
-            if target_attribute_group not in self.attributes:
-                self.attributes[target_attribute_group] = {}
-            target = self.attributes[target_attribute_group]
+            if group not in self.attributes:
+                self.attributes[group] = {}
+            self.attributes[group][name] = attribute
+        return attribute
 
-        if source_attribute_group is not None:
-            assert source_attribute_group in source_device.attributes
-            source = source_device[source_attribute_group]
+    def add_attribute(self, name, attribute, group=None):
+        attribute_list = [name]
+        if group is None:
+            self.attributes[name] = attribute
         else:
-            source = source_device.attributes
+            attribute_list.insert(0, group)
+            if group not in self.attributes:
+                self.attributes[group] = {}
+            self.attributes[group][name] = attribute
 
-        if shared_attributes == ALL:
-            target.update(source)
+        attribute_exists = attribute.attribute_id in self.attributes_sharing_map
+        self.attributes_sharing_map[attribute.attribute_id] = attribute_list
+        if attribute_exists and self.device_server is not None:
+            header = "{}.{}".format(attribute.attribute_id, TOKEN_ATTRIBUTE_REPLACE)
+            self.device_server.push_message(header, attribute)
+        return attribute
+
+    def share_group(self, source_device, source_group, target_group, attribute_name_mask=None):
+        if attribute_name_mask is None:
+            attribute_name_mask = {}
+        if source_group is not None:
+            from_group = source_device.attributes[source_group]
         else:
-            for attribute in shared_attributes:
-                target[attribute] = source[attribute]
+            from_group = source_device.attributes
 
-        if source_device.device_id not in self.attributes_sharing_map:
-            self.attributes_sharing_map[source_device.device_id] = {}
+        for attribute_name, attribute in from_group.items():
+            self.add_attribute(attribute_name_mask.get(attribute_name, attribute_name), attribute, target_group)
 
-        self.attributes_sharing_map[source_device.device_id][source_attribute_group] = (
-            target_attribute_group, shared_attributes)
-
-    def share_exposed_method(self, source_device, source_method, shared_method_attributes=None):
+    def share_exposed_method(self, source_device, source_method, shared_method_name=None,
+                             shared_method_attributes=None):
         """
         Share exposed method with source_device.
         This is handy method when You share attributes of other device and You want to also expose it's methods.
@@ -189,7 +193,8 @@ class Device(Subject, YamlSerializable):
         """
         for method_name, method_parameters in source_device.exposed_methods:
             if method_name == source_method:
-                shared_method_name = "{}_{}".format(source_device.device_id, method_name)
+                if shared_method_name is None:
+                    shared_method_name = "{}_{}".format(source_device.device_id, method_name)
                 if shared_method_attributes is None:
                     shared_method_attributes = method_parameters
                 method = copy(getattr(source_device, method_name))
@@ -219,7 +224,7 @@ class Device(Subject, YamlSerializable):
         else:
             return self.allow_reconnect
 
-    def attach_attribute_callback(self, attribute, callback, max_update_rate=None):
+    def attach_attribute_callback(self, attribute, callback, max_update_rate=None, key_filter=None):
         """
         Attach callback for every change of attribute.
         If max_update_rate is specified that callback will be called only after
@@ -231,8 +236,18 @@ class Device(Subject, YamlSerializable):
         """
         assert callable(callback)
         try:
-            attribute = self._get(attribute)
-            attribute.attach_callback(callback, max_update_rate)
+            attribute_object = self.get_attribute(attribute)
+            attribute_object.attach_callback(callback, max_update_rate, key_filter)
+
+            try:
+                self.attribute_attach_map[attribute_object.attribute_id].append((attribute, callback))
+            except KeyError:
+                self.attribute_attach_map.update({attribute_object.attribute_id: [(attribute, callback)]})
+
+            if key_filter is not None:
+                callback(attribute_object[key_filter])
+            else:
+                callback(VALUE, attribute_object[VALUE])
         except (KeyError, AttributeError):
             self.logger.exception(u"Attribute {} is not defined".format(attribute))
 
@@ -245,12 +260,72 @@ class Device(Subject, YamlSerializable):
         """
         assert callable(callback)
         try:
-            attribute = self._get(attribute)
-            attribute.detach_callback(callback)
+            attribute_object = self.get_attribute(attribute)
+            attribute_object.detach_callback(callback)
+
+            try:
+                if (attribute, callback) in self.attribute_attach_map[attribute_object.attribute_id]:
+                    self.attribute_attach_map[attribute_object.attribute_id].remove((attribute, callback))
+            except KeyError:
+                pass # Callback does not exists
+
         except (KeyError, AttributeError):
             if self.logger is not None:
                 self.logger.exception(u"Attribute {} is not defined".format(attribute))
 
+    def get_attribute(self, attribute):
+        """
+        Get Attribute value
+        :param attribute: tuple, list, str
+        :param value: mixed
+        :param callback: callable
+        :return: None
+        """
+        try:
+            return self._get(Attribute.list_attribute(attribute))
+        except KeyError:
+            return None
+
+    def get_value(self, attribute, key=VALUE):
+        """
+        Get Attribute[key] value
+        :param attribute: tuple, list, str
+        :param value: mixed
+        :param callback: callable
+        :return: None
+        """
+        try:
+            return self.get_attribute(attribute)[key]
+        except (KeyError, TypeError):
+            return None
+
+    def get_raw_value(self, attribute, key=VALUE):
+        """
+        Get attribute[key] value without offset or factor.
+        Use returned value to feed into real hardware.
+        :param attribute: list, dict, str
+        :return: None
+        """
+        try:
+            attribute_object = self.get_attribute(attribute)
+            return attribute_object.remove_offset_factor(attribute_object[key])
+        except (KeyError, TypeError):
+            return None
+
+    def _get(self, attribute):
+        """
+        This gets Device attribute.
+        Use this function when You want to get attribute by tuple or list key.
+        Example: Device.get((ATTR_STATUS, VALUE))
+        Otherwise use faster method Device[ATTR_STATUS][VALUE]
+        :param attribute: list
+        :return: mixed
+        """
+        try:
+            return get_from_dict(self.attributes, attribute)
+        except KeyError:
+            return None
+
     def set_attribute(self, attribute, value, callback=None):
         """
         This sets Device attribute.
@@ -260,72 +335,68 @@ class Device(Subject, YamlSerializable):
         :param callback: callable
         :return: None
         """
-        set_token = 0
-        if attribute[-1] == VALUE:
-            device_attribute = self.get_attribute(attribute[:-1])
-            with SetFunction(device_attribute, callback):
-                self._set(attribute, value)
-                set_token = device_attribute.read_and_reset_token()
-                if not set_token and callback is not None:
-                    callback(attribute, value)
+        list_attribute = Attribute.list_attribute(attribute)
+        if list_attribute[-1] == VALUE:
+            return self.set_value(list_attribute[:-1], value, callback)
         else:
-            self._set(attribute, value)
+            return self._set(list_attribute, value, callback)
 
-        return set_token
-
-    def get_attribute(self, attribute):
+    def set_value(self, attribute, value, callback=None, key=VALUE):
         """
-        This gets Device attribute.
-        Re-implement it for client or any special use.
-        :param attribute: tuple, list, str
-        :return: mixed
+        Set attribute value.
+        :param attribute: list, dict, str
+        :param value: mixed
+        :return: None
         """
-        return self._get(attribute)
+        attribute_list = Attribute.list_attribute(attribute)
+        attribute_object = self.get_attribute(attribute_list)
+        with SetFunction(attribute_object, callback):
+            return self._set(attribute_list + [key], value, callback)
 
-    def _set(self, attribute, value):
+    def set_raw_value(self, attribute, value, callback=None, key=VALUE):
+        """
+        Set attribute value obtained from device.
+        Apply offset and factor before value is set.
+        :param attribute: list, dict, str
+        :param value: mixed
+        :return: None
+        """
+        list_attribute = Attribute.list_attribute(attribute)
+        attribute_object = self.get_attribute(list_attribute)
+        value = attribute_object.apply_offset_factor(value)
+        self._set(list_attribute + [key], value, callback)
+
+    def _set(self, attribute_list, value, callback=None):
         """
         This sets Device attribute.
         Use this function when You want to set attribute by tuple or list key.
         Example: Device.set((ATTR_STATUS, VALUE), STATUS_IDLE)
         Attribute value is pushed into server if Device is connected on any.
         To reduce amount of pushed attributes we check if value is different from previous one.
-        :param attribute: tuple, list, str
+        :param attribute: list
         :param value: mixed
         :return: None
         """
-        if isinstance(attribute, (tuple, list)):
-            current_value = get_from_dict(self.attributes, attribute)
-            set_in_dict(self.attributes, attribute, value)
-            if attribute[-1] == VALUE:
-                # Get new value possibly affected by offset and factor
-                value = get_from_dict(self.attributes, attribute)
+        attribute_object = get_from_dict(self.attributes, attribute_list[:-1])
+        if attribute_list[-1] == VALUE and attribute_object.with_set_function:
+            propagate_change = attribute_object.set_value_when_set_function
         else:
-            current_value = self.attributes[attribute]
-            self.attributes[attribute] = value
-            if attribute == VALUE:
-                # Get new value possibly affected by offset and factor
-                value = self.attributes[attribute]
+            propagate_change = True
 
-        if value != current_value and self.device_server is not None:
-            token = "{}.{}".format(TOKEN_ATTRIBUTE, ".".join(attribute))
-            self.device_server.push_message(self.device_id, (attribute, value), token)
+        current_value = attribute_object[attribute_list[-1]]
+        attribute_object[attribute_list[-1]] = value
 
-    def _get(self, attribute):
-        """
-        This gets Device attribute.
-        Use this function when You want to get attribute by tuple or list key.
-        Example: Device.get((ATTR_STATUS, VALUE))
-        Otherwise use faster method Device[ATTR_STATUS][VALUE]
-        :param attribute: tuple, list, str
-        :return: mixed
-        """
-        try:
-            if isinstance(attribute, (tuple, list)):
-                return get_from_dict(self.attributes, attribute)
-            else:
-                return self.attributes.get(attribute)
-        except (KeyError, TypeError):
-            return None
+        set_token = attribute_object.read_and_reset_token()
+        if not set_token and callable(callback):
+            callback(attribute_list, value)
+
+        if propagate_change:
+            value = attribute_object[attribute_list[-1]]
+            if value != current_value and self.device_server is not None:
+                header = "{}.{}".format(attribute_object.attribute_id, attribute_list[-1])
+                self.device_server.push_message(header, value)
+
+        return set_token
 
     def connect(self, *args):
         """
@@ -335,7 +406,7 @@ class Device(Subject, YamlSerializable):
         try:
             self.connecting = True
             self.connected = False
-            self.device_poller.add_connecting_device(self)
+            self.device_connection_poller.add_connecting_device(self)
             self.handle_connect_event()
         except DeviceError:
             self.logger.exception(u"Connection exception")
@@ -512,48 +583,6 @@ class Device(Subject, YamlSerializable):
         else:
             return False
 
-    def get_value(self, attribute):
-        """
-        Get attributes value
-        :param attribute: list, dict, str
-        :return: None
-        """
-        if isinstance(attribute, tuple):
-            return self._get(attribute + (VALUE,))
-        elif isinstance(attribute, list):
-            return self._get(attribute + [VALUE])
-        else:
-            return self._get((attribute, VALUE))
-
-    def get_value_and_unit(self, attribute):
-        """
-        Get attributes value
-        :param attribute: list, dict, str
-        :return: None
-        """
-        if isinstance(attribute, tuple):
-            value, unit = self._get(attribute + (VALUE,)), self._get(attribute + (UNIT,))
-        elif isinstance(attribute, list):
-            value, unit = self._get(attribute + [VALUE]), self._get(attribute + [UNIT])
-        else:
-            value, unit = self._get((attribute, VALUE)), self._get((attribute, UNIT))
-
-        return units.Quantity(value, unit)
-
-    def set_value(self, attribute, value):
-        """
-        Set attributes value
-        :param attribute: list, dict, str
-        :param value: mixed
-        :return: None
-        """
-        if isinstance(attribute, tuple):
-            self._set(attribute + (VALUE,), value)
-        elif isinstance(attribute, list):
-            self._set(attribute + [VALUE], value)
-        else:
-            self._set((attribute, VALUE), value)
-
     def set_status(self, status):
         """
         Set attributes status value.
@@ -649,6 +678,9 @@ class Device(Subject, YamlSerializable):
         :return:
         """
         self.disconnect()
+        if self.device_server is not None:
+            self.device_server.remove_device(self)
+
         if self.logger is not None:
             for handler in self.logger.handlers[:]:
                 handler.close()
@@ -657,6 +689,11 @@ class Device(Subject, YamlSerializable):
         if self.session is not None:
             self.session.remove_device(self)
 
+        for cb_tuples in self.attribute_attach_map.values():
+            for cb_tuple in cb_tuples:
+                self.detach_attribute_callback(*cb_tuple)
+
+        self.attribute_attach_map = None
         self.attributes = None
         self.exposed_methods = None
         self.token_generator = None
@@ -800,49 +837,84 @@ class Device(Subject, YamlSerializable):
 
     def _expose_methods_to_clients(self):
         """
-        Call this method if You wish to expose method decorated with @expose_method.
-        This method is not called by default for Device!!!
+        This method is called to expose all marked methods to the clients.
         :return: None
         """
         for method in dir(self):
             method = getattr(self, method)
-            if callable(method) and hasattr(method, "exposed_parameters"):
+            if callable(method) and hasattr(method,
+                                            "exposed_parameters") and method.__name__ not in self.exposed_methods:
                 self.exposed_methods.append((method.__name__, method.exposed_parameters))
 
     def wait_for_status(self, statuses, retry_timeout=1000, callback=None):
         """
         Wait until device change it's status in one defined in statuses.
         If callback is set, then method is executed in non blocking thread.
-        :param statuses: List of one or multiple statuses to check against
+        :param statuses: List o one or multiple statuses to check against
         :param retry_timeout: Maximum time in ms to wait for status
         :param callback: Function to call after check in non blocking execution
         :return: True if device is in one of statuses
         """
 
         def _wait(_callback=None):
-            return_value = False
+            success = True
             retry_counter = 0
-            while not self.in_statuses(statuses) and self.connected:
+            while not self.in_statuses(statuses):
+                if not self.connected:
+                    return False
                 time.sleep(0.05)
                 if retry_counter >= retry_timeout:
                     break
                 retry_counter += 50
             if _callback is not None:
-                _callback(return_value)
+                _callback(success)
             else:
-                return True
+                if not success:
+                    raise DeviceError(
+                        "Status was not set to {} within timeout of {} ms".format(statuses, retry_timeout))
+                else:
+                    return True
 
         if callback is None:
             return _wait()
         else:
             Thread(target=_wait, args=[callback]).start()
 
+    def wait_until_value_set(self, attribute, value, retry_timeout=1000, callback=None):
+
+        def _wait(_callback=None):
+            attribute_object = self.get_attribute(attribute)
+            success = True
+            retry_counter = 0
+            while attribute_object[VALUE] != value:
+                if not self.connected:
+                    return False
+                time.sleep(0.05)
+                if retry_counter >= retry_timeout:
+                    success = False
+                    break
+                retry_counter += 50
+            if _callback is not None:
+                _callback(success)
+            else:
+                if not success:
+                    raise DeviceError(
+                        "Value of {} was not set to {} within timeout of {} ms".format(attribute, value, retry_timeout))
+                else:
+                    return True
+
+        if callback is None:
+            return _wait()
+        else:
+            thread = Thread(target=_wait, args=[callback])
+            thread.start()
+            return thread
+
     def is_alive(self):
         if not self.connected or self.closing or self.response_error:
             return False
-
         time_diff = self.request_timestamp - self.response_timestamp
         if time_diff >= self.response_timeout * 1e-3:
-            if not self.retry_command():
-                print(self.commands_buffer, self.commands_buffer)
-                self.handle_response_error(u"Device response timeout")
\ No newline at end of file
+            self.handle_response_error(u"Response timeout")
+            return False
+        return True
