@@ -1,16 +1,16 @@
 import json
 import time
 from threading import Thread, Event, Lock
+from time import sleep
 
 import zmq
+from zmq.utils.monitor import recv_monitor_message
 
-import kamzik3
 from kamzik3 import SEP, DeviceClientError
 from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
 from kamzik3.snippets.snippetDataAccess import get_from_dict, set_in_dict
-from kamzik3.snippets.snippetsControlLoops import control_device_client_poller
 from kamzik3.snippets.snippetsJson import JsonKamzikEncoder
 
 """ Example of yaml configuration
@@ -32,15 +32,17 @@ class DeviceClient(Device):
     initial_status = STATUS_DISCONNECTED
 
     def __init__(self, host, port, device_id=None, config=None):
+        self.subscription_thread = None
         Device.__init__(self, device_id, config)
         self.client_comm_lock = Lock()
-        self.device_poller = control_device_client_poller
         self.host = host
         self.port = port
         self.removing = False
         self.subscriber_port = None
         self.socket = None
         self.subscriber_socket = None
+        self.monitor_socket = None
+        self.monitor_thread = None
         self.callbacks = {}
         self.stopped = Event()
         self.subscribe_starts_with_list = []
@@ -63,15 +65,34 @@ class DeviceClient(Device):
         self.socket.setsockopt(zmq.RCVTIMEO, 3000)
         # Don't let socket linger after close
         # This is VERY IMPORTANT not to set to anything but 0
-        self.socket.setsockopt(zmq.LINGER, 0)
+        self.monitor_socket = self.socket.get_monitor_socket()
+
+        self.monitor_thread = Thread(target=self.connect_monitor_thread)
+        self.monitor_thread.start()
+
         try:
             self.socket.connect("tcp://{}:{}".format(self.host, self.port))
-            # Add socket to connection loop and return from connect method
-            self.device_poller.add_connecting_device(self)
         except zmq.ZMQError:
             self.handle_connection_error(u"Fatal connection error")
             return
 
+    def set_status(self, status):
+        self._set((ATTR_STATUS, VALUE), status)
+
+    def connect_monitor_thread(self):
+        while not self.stopped.is_set():
+            if self.monitor_socket.poll(timeout=500):
+                message = recv_monitor_message(self.monitor_socket)
+                if message["event"] == zmq.EVENT_CONNECTED:
+                    try:
+                        self.handle_connect_event()
+                    except DeviceClientError:
+                        # Server is ready, but device not yet configured
+                        if not self.stopped.wait(5):
+                            self.close()
+                            self.reconnect()
+                    return
+
     def handle_connect_event(self):
         """
         This method is called after we verified connection with main server.
@@ -91,24 +112,8 @@ class DeviceClient(Device):
             self.subscribe_starts_with_list = token
 
             # Check for attribute sharing and subscribe to them
-            if len(self.attributes_sharing_map) > 0:
-                self.subscribe_starts_with_list = [self.subscribe_starts_with_list]
-                for device_id, groups in self.attributes_sharing_map.items():
-                    token = self.get_token(TOKEN_ATTRIBUTE, device_id)
-                    for source_group, (target_group, source_attributes) in groups.items():
-                        if source_group != "null":
-                            sub_token = ".".join((token, source_group))
-                            self.subscribe_starts_with_list.append("{}.{}".format(token, source_group))
-                        else:
-                            sub_token = token
-                            self.subscribe_starts_with_list.append(sub_token)
-                        if source_attributes == ALL:
-                            self._subscribe(sub_token)
-                        else:
-                            for source_attribute in source_attributes:
-                                self._subscribe(".".join((sub_token, source_attribute)))
-                        self.subscribe_starts_with_list.append(sub_token)
-                self.subscribe_starts_with_list = tuple(self.subscribe_starts_with_list)
+            for attribute_id in self.attributes_sharing_map:
+                self._subscribe(attribute_id)
 
     def reconnect_allowed(self):
         if self.connected:
@@ -125,14 +130,15 @@ class DeviceClient(Device):
         self.logger.info(u"Initializing {} client subscriber thread".format(self.device_id))
         # Get SUB type of socker
         self.subscriber_socket = zmq.Context.instance().socket(zmq.SUB)
-        # Set receive timeout to 1000 ms
-        self.subscriber_socket.setsockopt(zmq.RCVTIMEO, 2000)
+        # Set receive timeout to 100 ms
+        self.subscriber_socket.setsockopt(zmq.RCVTIMEO, 100)
         # Don't let socket linger after close
         # This is VERY IMPORTANT not to set to anything but 0
         self.subscriber_socket.setsockopt(zmq.LINGER, 0)
         self.subscriber_socket.connect("tcp://{}:{}".format(self.host, self.subscriber_port))
         # Start separate Thread to read data from publisher
-        Thread(target=self._subscriber_thread).start()
+        self.subscription_thread = Thread(target=self._subscriber_thread)
+        self.subscription_thread.start()
 
     def _subscriber_thread(self):
         """
@@ -140,21 +146,22 @@ class DeviceClient(Device):
         Interrupt every zmq.RCVTIMEO and check if client is connected.
         :return: None
         """
+        # Check for published messages until client is connected
         while not self.stopped.isSet():
-            try:
+            if self.subscriber_socket.poll(timeout=100):
+                # Try to read any message from publisher
                 token, data = self.subscriber_socket.recv().decode().split(SEP)
                 data = json.loads(data)
                 self.handle_readout(token, data)
-            except zmq.Again:
-                # This exception is raised due mq.RCVTIMEO set to 1000ms.
-                # We interrupt recv loop to check if devices is still connected
-                # If not socket will be cleanly closed
-                if self.connected and not self.client_comm_lock.locked():
-                    self.poll()
-            except AttributeError:
-                pass  # Socket is not defined
+            elif self.monitor_socket.poll(timeout=0):
+                # Check if there is any message to read from monitor socket
+                message = recv_monitor_message(self.monitor_socket)
+                if message["event"] == zmq.EVENT_DISCONNECTED:
+                    self.handle_response_error(u"Socket disconnected")
+                    break
 
-        self.handle_closing_subscriber()
+        # Client was disconnected for any reason, finish cleaning up and possibly reconnect
+        self.handle_disconnection()
 
     def _subscribe(self, topic):
         """
@@ -213,24 +220,26 @@ class DeviceClient(Device):
             del self.callbacks[token]
 
     def handle_readout(self, token, data):
-        if token.startswith(self.subscribe_starts_with_list):
-            source_device_id = token.split(".", 1)[0]
-            if source_device_id != self.device_id:
-                group = data[0][:-1][0]
-                try:
-                    (target_group, shared_attributes) = self.attributes_sharing_map[source_device_id][group]
-                except KeyError:
-                    group = "null"
-                    (target_group, shared_attributes) = self.attributes_sharing_map[source_device_id][group]
-                if group is "null" and target_group is not "null":
-                    data[0].insert(0, target_group)
-                elif group is not "null" and target_group is "null":
-                    del data[0][0]
-                else:
-                    data[0][0] = target_group
-            self._set(*data)
-        elif token in self.callbacks:
+        if token in self.callbacks:
             self.handle_readout_callback(token, data)
+        else:
+            attribute_parts = token.split(".")
+            attribute_id = ".".join(attribute_parts[:-1])
+            if attribute_parts[1] == TOKEN_ATTRIBUTE:
+                if attribute_parts[-1] == TOKEN_ATTRIBUTE_REPLACE:
+                    attribute_path = attribute_parts[2:-1]
+                    attribute_path = self.attributes_sharing_map.get(attribute_id, attribute_path)
+                    self._set(attribute_path, Attribute.from_dict(data))
+                else:
+                    attribute_path = attribute_parts[2:]
+
+                    if attribute_id in self.attributes_sharing_map:
+                        self._set(self.attributes_sharing_map[attribute_id] + [attribute_parts[-1]], data)
+                    if token.startswith(self.subscribe_starts_with_list):
+                        self._set(attribute_path, data)
+            else:
+                raise DeviceClientError(
+                    "Unrecognized token part {}. Token value was: {}".format(attribute_parts[1], token))
 
     def init(self):
         """
@@ -247,7 +256,7 @@ class DeviceClient(Device):
                 self.subscriber_port, attributes, self.attributes_sharing_map, self.exposed_methods, self.qualified_name = json.loads(
                     response)
                 self.initial_status = attributes[ATTR_STATUS][VALUE]
-                self._sync_attributes(attributes)
+                self._sync_attributes(attributes, self.attributes)
                 for method_name, attributes in self.exposed_methods[:]:
                     setattr(self, method_name,
                             lambda method_name=method_name, **kwargs: self.method(method_name, kwargs))
@@ -261,7 +270,7 @@ class DeviceClient(Device):
             self.reconnect()
             return False
 
-    def _sync_attributes(self, synced_attributes):
+    def _sync_attributes(self, synced_attributes, parent_attribute):
         """
         synchronize new attributes with already existing attributes.
         This is important to prevent creating new attribute objects.
@@ -271,36 +280,37 @@ class DeviceClient(Device):
         """
         # Walk thru all items of dictionary
         for key, attribute in synced_attributes.items():
-            self._sync_attribute(key, attribute)
+            self._sync_attribute(key, attribute, parent_attribute)
 
-    def _sync_attribute(self, key, attribute):
+    def _sync_attribute(self, key, attribute, parent_attribute):
         """
         synchronize one attribute from new dict to existing self.attributes
         :param key: string
         :param attribute: dict
         :return:
         """
-        if key in self.attributes:
+        if key in parent_attribute:
             if Attribute.is_attribute(attribute):
-                if key == ATTR_STATUS:
+                if key == ATTR_STATUS and parent_attribute == self.attributes:
                     return
-                # Key exists in self.attributes
-                # Prevent attribute changes when syncing with new values
-                # with self.attributes[key].update_lock:
+                """
+                Key exists in self.attributes
+                Prevent attribute changes when syncing with new values
+                """
                 for attr_key in attribute.keys():
-                    if self.attributes[key][attr_key] != attribute[attr_key]:
-                        self.attributes[key][attr_key] = attribute[attr_key]
+                    if parent_attribute[key][attr_key] != attribute[attr_key]:
+                        parent_attribute[key][attr_key] = attribute[attr_key]
             else:
                 # Attribute is only sub group, continue syncing deeper
-                self._sync_attributes(attribute)
+                self._sync_attributes(attribute, parent_attribute[key])
         else:
             # Key des not exists in self.attributes
             if Attribute.is_attribute(attribute):
                 # Create completely new Attribute under specified Key
-                self.attributes[key] = Attribute.from_dict(attribute)
+                parent_attribute[key] = Attribute.from_dict(attribute)
             else:
                 # Attribute is only sub group, continue syncing deeper
-                self.attributes[key] = Attribute.from_dict(attribute)
+                parent_attribute[key] = Attribute.from_dict(attribute)
 
     def get_token(self, topic, device_id=None):
         return "{}.{}".format(self.device_id if device_id is None else device_id, topic)
@@ -337,7 +347,28 @@ class DeviceClient(Device):
             self.handle_response_error(u"Set error")
             return False
 
-    def _set(self, attribute, value):
+    def set_value(self, attribute, value, callback=None):
+        """
+        Set attribute value.
+        :param attribute: list, dict, str
+        :param value: mixed
+        :return: None
+        """
+        attribute_list = Attribute.list_attribute(attribute)
+        return self.set_attribute(attribute_list + [VALUE], value, callback)
+
+    def set_raw_value(self, attribute, value):
+        """
+        Set attribute value obtained from device.
+        Apply offset and factor before value is set.
+        :param attribute: list, dict, str
+        :param value: mixed
+        :return: None
+        """
+        raise DeviceClientError(u"Set raw value is not implemented on Client."
+                                u"Use set_value or set_attribute instead.")
+
+    def _set(self, attribute, value, callback=None):
         """
         This sets Device attribute.
         Use this function when You want to set attribute by tuple or list key.
@@ -349,9 +380,8 @@ class DeviceClient(Device):
         :return: None
         """
         current_value = get_from_dict(self.attributes, attribute)
-        if value == current_value:
-            return
-        set_in_dict(self.attributes, attribute, value)
+        if value != current_value:
+            set_in_dict(self.attributes, attribute, value)
 
     def poll(self):
         """
@@ -442,22 +472,10 @@ class DeviceClient(Device):
         start_at = time.time()
         self._config_attributes()
         self._config_commands()
+        self.set_status(STATUS_CONFIGURED)
         self.set_status(self.initial_status)
         self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))
 
-    def handle_closing_subscriber(self):
-        if self.subscriber_socket:
-            self.subscriber_socket.close()
-        if self.logger is not None:
-            self.logger.info(u"Client closed")
-        if not self.removing:
-            self.set_status(STATUS_DISCONNECTED)
-            self.closing = False
-            self.connected = False
-
-            if self.response_error or self.connection_error:
-                self.reconnect()
-
     def handle_response_error(self, message=None):
         self.logger.error(message)
         self.response_error = True
@@ -473,26 +491,25 @@ class DeviceClient(Device):
             u"Command error\nCommand: {}\nOutput: {}".format(readout_command, readout_output))
 
     def disconnect(self):
-        if self.connecting:
-            # Device is connecting, but here we want to disconnect it.
-            # To prevent automatic reconnection, just remove it from connecting devices.
-            self.device_poller.remove_connecting_device(self)
-        else:
-            # Device is fully connected and it's not in connecting loop
-            # Therefor we can safely close it
-            self.close()
+        self.close()
 
     def close(self):
-        self.logger.info(u"Closing client")
+        if self.logger is not None:
+            self.logger.info(u"Closing client")
         self.closing = True
-        self.connected = False
+        self.socket.disable_monitor()
         self.set_status(STATUS_DISCONNECTING)
-        if self.socket:
-            self.socket.close()
         self.stopped.set()
-        # Here we doing nothing more
-        # After stopped flag is set handle_closing_subscriber is fired and finish
-        # client closing procedure
+
+    def handle_disconnection(self):
+        if self.logger is not None:
+            self.logger.info(u"Client closed")
+        if not self.removing:
+            self.closing = False
+            self.connected = False
+            self.set_status(STATUS_DISCONNECTED)
+            if self.response_error or self.connection_error:
+                self.reconnect()
 
     def remove(self):
         self.removing = True
