@@ -7,7 +7,6 @@ import numpy as np
 import serial
 
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
 from kamzik3.devices.devicePort import DevicePort
 from kamzik3.devices.deviceSocket import DeviceSocket
@@ -42,7 +41,31 @@ class DeviceGalil(DeviceSocket):
              ("AO5", np.ushort), ("AO6", np.ushort), ("AO7", np.ushort)])
         self.preset_io_attributes()
         super().__init__(host, port, device_id, config)
-        self._expose_methods_to_clients()
+
+    def _init_attributes(self):
+        super(DeviceGalil, self)._init_attributes()
+
+        self.create_attribute(ATTR_MODEL, default_value="", readonly=True)
+
+        self["Analog input"] = {}
+        self["Analog output"] = {}
+        self["Digital input"] = {}
+        self["Digital output"] = {}
+
+        for i in range(8):
+            self.create_attribute(self.AI_config[i], "Analog input", min_value=0., max_value=10., unit=u"V",
+                                  default_type=np.float16, decimals=4, readonly=True)
+            self.create_attribute(self.AO_config[i], "Analog output", min_value=0., max_value=10., unit=u"V",
+                                  set_function=lambda value, output=i: self.set_analog_output(output, value),
+                                  default_type=np.float16, decimals=4)
+            self.create_attribute(self.DI_config[i], "Digital input", default_type=np.bool, readonly=True)
+            self.create_attribute(self.DI_config[8 + i], "Digital input", default_type=np.bool, readonly=True)
+            self.create_attribute(self.DO_config[i], "Digital output", default_type=np.bool,
+                                  set_value_when_set_function=False,
+                                  set_function=lambda value, output=i: self.set_digital_output(output, value))
+            self.create_attribute(self.DO_config[8 + i], "Digital output", default_type=np.bool,
+                                  set_value_when_set_function=False,
+                                  set_function=lambda value, output=8 + i: self.set_digital_output(output, value))
 
     def preset_io_attributes(self):
         self.DO_config = [
@@ -99,7 +122,7 @@ class DeviceGalil(DeviceSocket):
                 self.response_timestamp = time.time()
             except timeout:
                 self.request_timestamp = time.time()
-                DevicePoller.check_device_alive(self)
+                # DevicePoller.check_device_alive(self)
             except OSError:
                 # Socket doesn't exist
                 pass
@@ -113,13 +136,12 @@ class DeviceGalil(DeviceSocket):
         try:
             model_name, revision = self.get_value(ATTR_MODEL).split(" ", maxsplit=1)
             if model_name == u"RIO47100":
-                sample_number = np.frombuffer(data[4:6], dtype=np.ushort)
                 for index, ao in enumerate(np.frombuffer(data[8:24], dtype=self.analog_output_date_type)[0]):
                     ao_attribute = self["Analog output"][self.AO_config[index]]
                     ao_min, ao_max = ao_attribute.remove_offset_factor(
                         ao_attribute[MIN]), ao_attribute.remove_offset_factor(ao_attribute[MAX])
                     dt_info = np.iinfo(self.analog_output_date_type[index])
-                    self.set_value(("Analog output", self.AO_config[index]),
+                    self.set_raw_value(("Analog output", self.AO_config[index]),
                                    (((ao - dt_info.min) * (ao_max - ao_min)) / (dt_info.max - dt_info.min)) + ao_min)
 
                 for index, ai in enumerate(np.frombuffer(data[24:40], dtype=self.analog_input_date_type)[0]):
@@ -127,7 +149,7 @@ class DeviceGalil(DeviceSocket):
                     ai_min, ai_max = ai_attribute.remove_offset_factor(
                         ai_attribute[MIN]), ai_attribute.remove_offset_factor(ai_attribute[MAX])
                     dt_info = np.iinfo(self.analog_input_date_type[index])
-                    self.set_value(("Analog input", self.AI_config[index]),
+                    self.set_raw_value(("Analog input", self.AI_config[index]),
                                    (((ai - dt_info.min) * (ai_max - ai_min)) / (dt_info.max - dt_info.min)) + ai_min)
 
                 digital_output = format(np.frombuffer(data[40:42], dtype=np.ushort)[0], "b")
@@ -137,40 +159,12 @@ class DeviceGalil(DeviceSocket):
                 digital_input = [bool(int(x)) for x in reversed(digital_input)] + ([False] * (16 - len(digital_input)))
 
                 for i, do in enumerate(digital_output):
-                    self.set_value(["Digital output", self.DO_config[i]], do)
+                    self.set_raw_value(["Digital output", self.DO_config[i]], do)
                 for i, di in enumerate(digital_input):
-                    self.set_value(["Digital input", self.DI_config[i]], di)
+                    self.set_raw_value(["Digital input", self.DI_config[i]], di)
         except ValueError:
             pass
 
-    def _init_attributes(self):
-        super(DeviceGalil, self)._init_attributes()
-
-        self["Analog input"] = {}
-        self["Analog output"] = {}
-        self["Digital input"] = {}
-        self["Digital output"] = {}
-
-        for i in range(8):
-            self["Analog input"][self.AI_config[i]] = Attribute(min_value=0., max_value=10., unit=u"V",
-                                                                default_type=np.float16, decimals=4, readonly=True)
-
-            attribute = Attribute(min_value=0., max_value=10., unit=u"V",
-                                  set_function=lambda value, output=i: self.set_analog_output(output, value),
-                                  default_type=np.float16, decimals=4)
-            self["Analog output"][self.AO_config[i]] = attribute
-            self["Digital input"][self.DI_config[i]] = Attribute(default_type=np.bool, readonly=True)
-            self["Digital input"][self.DI_config[8 + i]] = Attribute(default_type=np.bool, readonly=True)
-            attribute = Attribute(default_type=np.bool,
-                                  set_function=lambda value, output=i: self.set_digital_output(output, value))
-            self["Digital output"][self.DO_config[i]] = attribute
-            attribute = Attribute(default_type=np.bool,
-                                  set_value_when_set_function=False,
-                                  set_function=lambda value, output=8 + i: self.set_digital_output(output, value))
-            self["Digital output"][self.DO_config[8 + i]] = attribute
-
-        self[ATTR_MODEL] = Attribute("", readonly=True)
-
     def start_polling(self):
         if self.udp_port is None:
             Device.start_polling(self)
@@ -190,20 +184,20 @@ class DeviceGalil(DeviceSocket):
         elif command == self.analog_input_readout_command:
             analog_inputs = map(float, output.split())
             for i, value in enumerate(analog_inputs):
-                self.set_value(("Analog input", self.AI_config[i]), value)
+                self.set_raw_value(("Analog input", self.AI_config[i]), value)
         elif command == self.analog_output_readout_command:
             analog_outputs = map(float, output.split())
             for i, value in enumerate(analog_outputs):
-                self.set_value(("Analog output", self.AO_config[i]), value)
+                self.set_raw_value(("Analog output", self.AO_config[i]), value)
         elif command == self.digital_lO_readout_command:
             di0, di1, do0, do1 = map(lambda v: int(float(v)), output.split())
             for bit in range(8):
-                self.set_value(("Digital input", self.DI_config[bit]), bool(di0 & (1 << bit)))
-                self.set_value(("Digital input", self.DI_config[8 + bit]), bool(di1 & (1 << bit)))
-                self.set_value(("Digital output", self.DO_config[bit]), bool(do0 & (1 << bit)))
-                self.set_value(("Digital output", self.DO_config[8 + bit]), bool(do1 & (1 << bit)))
+                self.set_raw_value(("Digital input", self.DI_config[bit]), bool(di0 & (1 << bit)))
+                self.set_raw_value(("Digital input", self.DI_config[8 + bit]), bool(di1 & (1 << bit)))
+                self.set_raw_value(("Digital output", self.DO_config[bit]), bool(do0 & (1 << bit)))
+                self.set_raw_value(("Digital output", self.DO_config[8 + bit]), bool(do1 & (1 << bit)))
         elif command == "\x12\x16;":
-            self.set_value(ATTR_MODEL, output)
+            self.set_raw_value(ATTR_MODEL, output)
         elif command == self.analog_output_limits_command:
             analog_outputs_limits = output.strip().split()
             # This is analog output channels limits mapping from Galil doc
@@ -331,7 +325,6 @@ class DeviceGalilPort(DevicePort, DeviceGalil):
              ("AO5", np.ushort), ("AO6", np.ushort), ("AO7", np.ushort)])
         self.preset_io_attributes()
         DevicePort.__init__(self, port, baud_rate, parity, stop_bits, byte_size, device_id, config)
-        self._expose_methods_to_clients()
 
     def handshake(self):
         self.logger.info(u"Handshake initiated")
