@@ -7,14 +7,13 @@ from bidict import frozenbidict
 
 from kamzik3 import DeviceError, units
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.deviceGalil import DeviceGalil
 from kamzik3.devices.devicePort import DevicePort
+from kamzik3.macro.step import StepDeviceAttributeNumerical
 from kamzik3.snippets.snippetsDecorators import expose_method
 from kamzik3.snippets.snippetsUnits import device_units
-from kamzik3.macro.step import StepDeviceAttributeNumerical
 
 MOTOR_TYPE_PWM_SIGN_SERVO = u"PMW sign servo (1.5)"
 MOTOR_TYPE_PWM_SIGN_SERVO_REVERSED_POLARITY = u"PMW servo reversed polarity (-1.5)"
@@ -68,6 +67,17 @@ class DeviceGalilMotor(DeviceGalil):
     push_commands_max = 1
     response_timeout = 3000
 
+    def _init_attributes(self):
+        Device._init_attributes(self)
+
+        self.create_attribute(ATTR_CHANNELS, readonly=True, default_type=np.uint8, min_value=0, max_value=256)
+        self.create_attribute(ATTR_LIMIT_SWITCH_ACTIVE_LEVEL, default_type=LIMIT_SWITCH_LEVELS.keys(),
+                              set_function=self.set_limit_switch_active_level)
+        self.create_attribute(ATTR_HOMING_DIRECTION, default_type=HOMING_DIRECTION.keys(),
+                              set_function=self.set_homing_direction)
+        self.create_attribute(ATTR_LATCH_INPUT_LEVEL, default_type=LIMIT_SWITCH_LEVELS.keys(),
+                              set_function=self.set_latch_input_level)
+
     def handle_configuration(self):
         start_at = time.time()
 
@@ -83,18 +93,6 @@ class DeviceGalilMotor(DeviceGalil):
         self.command(u"QZ;")
         self.get_motor_config(callback=_finish_configuration)
 
-    def _init_attributes(self):
-        Device._init_attributes(self)
-        self.attributes.update({
-            ATTR_CHANNELS: Attribute(None, readonly=True, default_type=np.uint8, min_value=0, max_value=256),
-            ATTR_LIMIT_SWITCH_ACTIVE_LEVEL: Attribute(None, default_type=LIMIT_SWITCH_LEVELS.keys(),
-                                                      set_function=self.set_limit_switch_active_level),
-            ATTR_HOMING_DIRECTION: Attribute(None, default_type=HOMING_DIRECTION.keys(),
-                                             set_function=self.set_homing_direction),
-            ATTR_LATCH_INPUT_LEVEL: Attribute(None, default_type=LIMIT_SWITCH_LEVELS.keys(),
-                                              set_function=self.set_latch_input_level),
-        })
-
     def start_polling(self):
         Device.start_polling(self)
         self.poll_command(u"TD;", 300)
@@ -120,14 +118,14 @@ class DeviceGalilMotor(DeviceGalil):
         elif command == u"MG _CN0, _CN1, _CN2, _CN3, _CN4;":
             ls_level, home_direction, latch_input_level, selective_abort, no_termniation_flag = [int(float(v)) for v in
                                                                                                  output.split(" ")]
-            self.set_value(ATTR_LIMIT_SWITCH_ACTIVE_LEVEL, LIMIT_SWITCH_LEVELS.inv.get(ls_level))
-            self.set_value(ATTR_HOMING_DIRECTION, HOMING_DIRECTION.inv.get(home_direction))
-            self.set_value(ATTR_LATCH_INPUT_LEVEL, LIMIT_SWITCH_LEVELS.inv.get(latch_input_level))
+            self.set_raw_value(ATTR_LIMIT_SWITCH_ACTIVE_LEVEL, LIMIT_SWITCH_LEVELS.inv.get(ls_level))
+            self.set_raw_value(ATTR_HOMING_DIRECTION, HOMING_DIRECTION.inv.get(home_direction))
+            self.set_raw_value(ATTR_LATCH_INPUT_LEVEL, LIMIT_SWITCH_LEVELS.inv.get(latch_input_level))
         elif command == u"QZ;":
             parsed_output = [int(float(v)) for v in output.split(", ")]
-            self.set_value(ATTR_CHANNELS, parsed_output[0])
+            self.set_raw_value(ATTR_CHANNELS, parsed_output[0])
         # elif command == u"MG _QZ0;":
-        #     self.set_value(ATTR_CHANNELS, int(float(output)))
+        #     self.set_raw_value(ATTR_CHANNELS, int(float(output)))
         elif command == u"TC 1;":
             readout_buffer = output
             self.handle_command_error(command, readout_buffer)
@@ -209,6 +207,51 @@ class DeviceGalilMotorChannel(DeviceChannel):
         self.final_position_setpoint = None
         DeviceChannel.__init__(self, device, channel, device_id, config)
 
+    def _init_attributes(self):
+        super(DeviceGalilMotorChannel, self)._init_attributes()
+
+        self.create_attribute(ATTR_ACRONYM, default_value=self.acronym, readonly=True)
+        self.create_attribute(ATTR_AMPLIFIER_GAIN, default_type=np.uint8, min_value=0, max_value=3,
+                              set_function=self.set_amplifier_gain)
+        self.create_attribute(ATTR_MOTOR_TYPE, default_type=MOTOR_TYPES.keys(), set_function=self.set_motor_type)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.float, min_value=-15e6, max_value=15e6, unit=u"steps/sec",
+                              set_function=self.set_velocity)
+        self.create_attribute(ATTR_STEP_DRIVE_RESOLUTION, default_type=np.uint64, max_value=15e6, unit=u"ustep",
+                              set_function=self.set_step_drive_resolution)
+        self.create_attribute(ATTR_BACKLASH_DISTANCE, default_value=0, default_type=np.float64, readonly=False,
+                              decimals=3)
+        self.create_attribute(FLAG_FORWARD_LIMIT_INACTIVE, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_HOME_SWITCH, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_MOTOR_MOVING, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_MOTOR_OFF, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_POSITION_EXCEEDS_ERROR, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_POSITION_LATCH, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_REFERENCING, default_type=np.bool, readonly=True)
+        self.create_attribute(FLAG_REVERSE_LIMIT_INACTIVE, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_LOW_CURRENT_STEPPER_MODE, default_type=np.uint32, max_value=32767,
+                              set_function=self.set_low_current_stepper_mode)
+        self.create_attribute(ATTR_ACCELERATION, default_type=np.uint64, min_value=1024, max_value=1073740800,
+                              set_function=self.set_acceleration)
+        self.create_attribute(ATTR_DECELERATION, default_type=np.uint64, min_value=1024, max_value=1073740800,
+                              set_function=self.set_deceleration)
+        self.create_attribute(ATTR_POSITION, default_type=np.float64, unit=u"counts", set_function=self._move_absolute,
+                              set_value_when_set_function=False)
+        self.create_attribute(ATTR_HOMING_METHOD, default_value=u"Limit based", default_type=HOMING_METHODS.keys())
+        self.create_attribute(ATTR_LIMIT_SWITCHES_MODE, default_type=LIMIT_SWITCHES_MODE.keys(),
+                              set_function=self.set_limit_switch_mode)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.float64, unit=u"counts",
+                              set_function=self.set_lower_position_limit)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.float64, unit=u"counts",
+                              set_function=self.set_upper_position_limit)
+        self.create_attribute(ATTR_FAST_REFERENCE_VELOCITY, default_value=5, default_type=np.float, min_value=-15e6,
+                              max_value=15e6, unit=u"steps/sec")
+        self.create_attribute(ATTR_SLOW_REFERENCE_VELOCITY, default_value=1, default_type=np.float, min_value=-15e6,
+                              max_value=15e6, unit=u"steps/sec")
+        self.create_attribute(ATTR_SAFE_DIRECTION, default_value=u"Forward", default_type=HOMING_DIRECTION.keys())
+        self.create_attribute(ATTR_REFERENCED, default_value=False, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_REFERENCE_STEPS_BACK, default_value=40, default_type=np.uint32, min_value=1,
+                              unit=u"steps")
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -232,50 +275,6 @@ class DeviceGalilMotorChannel(DeviceChannel):
         self.get_limit_switch_mode()
         self.get_amplifier_gain(callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DeviceGalilMotorChannel, self)._init_attributes()
-        self.attributes.update({
-            ATTR_ACRONYM: Attribute(self.acronym, readonly=True),
-            ATTR_AMPLIFIER_GAIN: Attribute(None, default_type=np.uint8, min_value=0, max_value=3,
-                                           set_function=self.set_amplifier_gain),
-            ATTR_MOTOR_TYPE: Attribute(None, default_type=MOTOR_TYPES.keys(), set_function=self.set_motor_type),
-            ATTR_VELOCITY: Attribute(None, default_type=np.float, min_value=-15e6, max_value=15e6, unit=u"steps/sec",
-                                     set_function=self.set_velocity),
-            ATTR_STEP_DRIVE_RESOLUTION: Attribute(None, default_type=np.uint64, max_value=15e6, unit=u"ustep",
-                                                  set_function=self.set_step_drive_resolution),
-            ATTR_BACKLASH_DISTANCE: Attribute(0, default_type=np.float64, readonly=False, decimals=3),
-            FLAG_FORWARD_LIMIT_INACTIVE: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_HOME_SWITCH: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_MOTOR_MOVING: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_MOTOR_OFF: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_POSITION_EXCEEDS_ERROR: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_POSITION_LATCH: Attribute(None, default_type=np.bool, readonly=True),
-            FLAG_REFERENCING: Attribute(False, default_type=np.bool, readonly=True),
-            FLAG_REVERSE_LIMIT_INACTIVE: Attribute(None, default_type=np.bool, readonly=True),
-            ATTR_LOW_CURRENT_STEPPER_MODE: Attribute(None, default_type=np.uint32, max_value=32767,
-                                                     set_function=self.set_low_current_stepper_mode),
-            ATTR_ACCELERATION: Attribute(None, default_type=np.uint64, min_value=1024, max_value=1073740800,
-                                         set_function=self.set_acceleration),
-            ATTR_DECELERATION: Attribute(None, default_type=np.uint64, min_value=1024, max_value=1073740800,
-                                         set_function=self.set_deceleration),
-            ATTR_POSITION: Attribute(None, default_type=np.float64, unit=u"counts", set_function=self._move_absolute,
-                                     set_value_when_set_function=False),
-            ATTR_HOMING_METHOD: Attribute(u"Limit based", default_type=HOMING_METHODS.keys()),
-            ATTR_LIMIT_SWITCHES_MODE: Attribute(None, default_type=LIMIT_SWITCHES_MODE.keys(),
-                                                set_function=self.set_limit_switch_mode),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.float64, unit=u"counts",
-                                                 set_function=self.set_lower_position_limit),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.float64, unit=u"counts",
-                                                 set_function=self.set_upper_position_limit),
-            ATTR_FAST_REFERENCE_VELOCITY: Attribute(5, default_type=np.float, min_value=-15e6, max_value=15e6,
-                                                    unit=u"steps/sec"),
-            ATTR_SLOW_REFERENCE_VELOCITY: Attribute(1, default_type=np.float, min_value=-15e6, max_value=15e6,
-                                                    unit=u"steps/sec"),
-            ATTR_SAFE_DIRECTION: Attribute(u"Forward", default_type=HOMING_DIRECTION.keys()),
-            ATTR_REFERENCED: Attribute(False, default_type=np.bool, readonly=True),
-            ATTR_REFERENCE_STEPS_BACK: Attribute(40, default_type=np.uint32, min_value=1, unit=u"steps"),
-        })
-
     def subject_update(self, key, value, subject):
         super(DeviceGalilMotorChannel, self).subject_update(key, value, subject)
 
@@ -283,7 +282,7 @@ class DeviceGalilMotorChannel(DeviceChannel):
             attribute = key[1]
 
             if attribute == ATTR_VELOCITY:
-                value /= self.get_value(ATTR_STEP_DRIVE_RESOLUTION)
+                value /= self.get_raw_value(ATTR_STEP_DRIVE_RESOLUTION)
             elif attribute == ATTR_STATUS:
                 if self.get_value(FLAG_REFERENCING):
                     value = STATUS_BUSY
@@ -291,8 +290,8 @@ class DeviceGalilMotorChannel(DeviceChannel):
                         FLAG_MOTOR_OFF) and self.get_value(ATTR_LOW_CURRENT_STEPPER_MODE) == 0:
                     self.servo_off()
 
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
     def _move_absolute(self, position):
         self.servo_on()
@@ -333,13 +332,13 @@ class DeviceGalilMotorChannel(DeviceChannel):
     def move_relative(self, position):
         position = device_units(self, ATTR_POSITION, position)
         self.logger.info(u"Move motor channel by relative position {:~}".format(position))
-        return self.set_attribute([ATTR_POSITION, VALUE], self[ATTR_POSITION][VALUE] + position.m)
+        return self.set_value(ATTR_POSITION, self[ATTR_POSITION][VALUE] + position.m)
 
     @expose_method({"position": ATTR_POSITION})
     def move_absolute(self, position):
         position = device_units(self, ATTR_POSITION, position)
         self.logger.info(u"Move motor channel to absolute position {:~}".format(position))
-        return self.set_attribute([ATTR_POSITION, VALUE], position.m)
+        return self.set_value(ATTR_POSITION, position.m)
 
     def set_limit_switch_mode(self, value, callback=None):
         return self.command(u"LD{}={};".format(self.acronym, LIMIT_SWITCHES_MODE.get(value)), callback, with_token=True)
@@ -368,13 +367,11 @@ class DeviceGalilMotorChannel(DeviceChannel):
 
     def set_lower_position_limit(self, value, callback=None):
         self.set_value(ATTR_POSITION_LOWER_LIMIT, value)
-        lower_position_limit = self[ATTR_POSITION].remove_offset_factor(value)
-        return self.set_attribute([ATTR_POSITION, MIN], lower_position_limit, callback)
+        return self.set_attribute([ATTR_POSITION, MIN], value, callback)
 
     def set_upper_position_limit(self, value, callback=None):
         self.set_value(ATTR_POSITION_UPPER_LIMIT, value)
-        upper_position_limit = self[ATTR_POSITION].remove_offset_factor(value)
-        return self.set_attribute([ATTR_POSITION, MAX], upper_position_limit, callback)
+        return self.set_attribute([ATTR_POSITION, MAX], value, callback)
 
     def get_motor_type(self, callback=None):
         self.command(u"MG _MT{};".format(self.acronym), callback)
