@@ -7,7 +7,6 @@ from pint.errors import UndefinedUnitError
 
 from kamzik3 import DeviceUnitError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.deviceSocket import DeviceSocket
 from kamzik3.snippets.snippetsDecorators import expose_method
@@ -158,11 +157,9 @@ class DeviceKohzu(DeviceSocket):
     def _init_attributes(self):
         super(DeviceKohzu, self)._init_attributes()
 
-        self.attributes.update({
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_CHANNELS: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_MODULES: Attribute(None, readonly=True, default_type=np.uint16),
-        })
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_CHANNELS, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_MODULES, readonly=True, default_type=np.uint16)
 
     def valid_command_format(self, command):
         if command is not None and command[-1] == "\n":
@@ -237,6 +234,92 @@ class DeviceKohzu(DeviceSocket):
 
 class DeviceKohzuChannel(DeviceChannel):
 
+    def _init_attributes(self):
+        super(DeviceKohzuChannel, self)._init_attributes()
+
+        self.create_attribute(ATTR_STATE, readonly=True, default_type=np.uint8, min_value=0, max_value=3)
+        self.create_attribute(ATTR_EMERGENCY_STOP, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_CWL, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_CCWL, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_ORG, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_NORG, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_ROTATION_CHANGE, default_type=np.uint8, min_value=0, max_value=1,
+                              set_function=lambda v: self.set_sys(ATTR_ROTATION_CHANGE, v))
+        self.create_attribute(ATTR_SWAP_LIMITS, default_type=np.uint8, min_value=0, max_value=1,
+                              set_function=lambda v: self.set_sys(ATTR_SWAP_LIMITS, v))
+        self.create_attribute(ATTR_ENCODER_READ, default_type=np.int64, set_function=self.set_encoder_value)
+        self.create_attribute(ATTR_PULSE_READ, default_type=np.int64, set_function=self.set_pulse_value, unit="pulse")
+        self.create_attribute(ATTR_POSITION, default_type=np.int64, set_function=self._move_absolute, unit="nm",
+                              set_value_when_set_function=False)
+        self.create_attribute(ATTR_REFERENCING_METHOD, default_type=np.uint8, min_value=1, max_value=15,
+                              set_function=lambda v: self.set_sys(ATTR_REFERENCING_METHOD, v))
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.int64, min_value=-134217728,
+                              max_value=134217727, unit="nm",
+                              set_function=lambda v: self.set_sys(ATTR_POSITION_LOWER_LIMIT, v))
+        self.create_attribute(ATTR_STEP_RESOLUTION, default_value=500, default_type=np.int64, min_value=1,
+                              max_value=134217727, unit="nm")
+        self.create_attribute(ATTR_START_SPEED, default_value=1, default_type=np.int32, min_value=1, max_value=2.5e6,
+                              unit="pulse/s",
+                              set_function=lambda v: self.write_speed_table(start_speed=v))
+        self.create_attribute(ATTR_TOP_SPEED, default_value=1, default_type=np.int32, min_value=2, max_value=5e6,
+                              unit="pulse/s",
+                              set_function=lambda v: self.write_speed_table(top_speed=v))
+        self.create_attribute(ATTR_ACCELERATION_TIME, default_value=1, default_type=np.int32, min_value=1,
+                              max_value=10e3, unit="ms",
+                              set_function=lambda v: self.write_speed_table(acceleration_time=v))
+        self.create_attribute(ATTR_DECELERATION_TIME, default_value=1, default_type=np.int32, min_value=1,
+                              max_value=10e3, unit="ms",
+                              set_function=lambda v: self.write_speed_table(deceleration_time=v))
+        self.create_attribute(ATTR_ENCODER_RESOLUTION, default_value=50, default_type=np.int64, min_value=1,
+                              max_value=134217727, unit="nm")
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.int64, min_value=-134217728,
+                              max_value=134217727, unit="nm",
+                              set_function=lambda v: self.set_sys(ATTR_POSITION_UPPER_LIMIT, v))
+        self.create_attribute(ATTR_MAXIMUM_VELOCITY, default_type=np.uint32, min_value=2, max_value=5000000,
+                              set_function=lambda v: self.set_sys(ATTR_MAXIMUM_VELOCITY, v))
+        self.create_attribute(ATTR_ENCODER_CALC_NUM, default_type=np.uint32, min_value=1, max_value=134217727,
+                              set_function=lambda v: self.set_sys(ATTR_ENCODER_CALC_NUM, v))
+        self.create_attribute(ATTR_ENCODER_CALC_DEN, default_type=np.uint32, min_value=1, max_value=134217727,
+                              set_function=lambda v: self.set_sys(ATTR_ENCODER_CALC_DEN, v))
+        self.create_attribute(ATTR_FEEDBACK_CONTROL_TYPE, default_type=DeviceKohzu.feedback_control_method.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_FEEDBACK_CONTROL_TYPE,
+                                                                  DeviceKohzu.feedback_control_method[v]))
+        self.create_attribute(ATTR_RETRY_COUNT, default_type=np.uint16, min_value=1, max_value=10e3,
+                              set_function=lambda v: self.set_sys(ATTR_RETRY_COUNT, v))
+        self.create_attribute(ATTR_FEEDBACK_TIMEOUT, default_type=np.uint16, min_value=1, max_value=10e3, unit="ms",
+                              set_function=lambda v: self.set_sys(ATTR_FEEDBACK_TIMEOUT, v))
+        self.create_attribute(ATTR_TRIGGER_SOURCE, default_type=DeviceKohzu.trigger_source.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_SOURCE, DeviceKohzu.trigger_source[v]))
+        self.create_attribute(ATTR_TRIGGER_EDGE, default_type=DeviceKohzu.trigger_edge.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_EDGE, DeviceKohzu.trigger_edge[v]))
+        self.create_attribute(ATTR_TRIGGER_PM_PITCH, default_type=np.uint32, min_value=1, max_value=100e3,
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_PM_PITCH, v))
+        self.create_attribute(ATTR_TRIGGER_ENC_PITCH, default_type=np.uint32, min_value=1, max_value=100e3,
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_ENC_PITCH, v))
+        self.create_attribute(ATTR_TRIGGER_PULSE_WIDTH, default_type=DeviceKohzu.trigger_pulse_width.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_PULSE_WIDTH,
+                                                                  DeviceKohzu.trigger_pulse_width[v]))
+        self.create_attribute(ATTR_ENCODER_ADDING, default_type=DeviceKohzu.encoder_adding.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_ENCODER_ADDING, DeviceKohzu.encoder_adding[v]))
+        self.create_attribute(ATTR_TRIGGER_LOGIC, default_type=DeviceKohzu.trigger_logic.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_LOGIC, DeviceKohzu.trigger_logic[v]))
+        self.create_attribute(ATTR_MOTOR_EXCITATION, default_type=DeviceKohzu.motor_excitation.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_MOTOR_EXCITATION,
+                                                                  DeviceKohzu.motor_excitation[v]))
+        self.create_attribute(ATTR_MOTOR_SELECTION, default_type=DeviceKohzu.motor_selection.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_MOTOR_SELECTION, DeviceKohzu.motor_selection[v]))
+        self.create_attribute(ATTR_MICROSTEP_SELECT, default_type=DeviceKohzu.microstep_select.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_MICROSTEP_SELECT,
+                                                                  DeviceKohzu.microstep_select[v]))
+        self.create_attribute(ATTR_STOP_TYPE, default_type=DeviceKohzu.stop_type.keys(),
+                              set_function=lambda v: self.set_sys(ATTR_STOP_TYPE, DeviceKohzu.stop_type[v]))
+        self.create_attribute(ATTR_ACCELERATION_PATTERN, default_type=DeviceKohzu.accelerating_pattern.keys(),
+                              set_function=lambda v: self.write_speed_table(acceleration_pattern=v))
+        self.create_attribute(ATTR_READ_ENCODER, default_value=True, readonly=False, default_type=np.bool)
+        self.create_attribute(ATTR_VELOCITY, default_value=0, default_type=np.int64, min_value=0, max_value=3e6,
+                              unit="nm/s", set_function=self.set_velocity)
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -276,111 +359,12 @@ class DeviceKohzuChannel(DeviceChannel):
         self.get_referenced()
         self.command(u"RTB{}/0\r\n".format(self.channel + 1), callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DeviceKohzuChannel, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint8, min_value=0, max_value=3),
-            ATTR_EMERGENCY_STOP: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_CWL: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_CCWL: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_ORG: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_NORG: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_REFERENCED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_ROTATION_CHANGE: Attribute(None, default_type=np.uint8, min_value=0, max_value=1,
-                                            set_function=lambda v: self.set_sys(ATTR_ROTATION_CHANGE, v)),
-            ATTR_SWAP_LIMITS: Attribute(None, default_type=np.uint8, min_value=0, max_value=1,
-                                        set_function=lambda v: self.set_sys(ATTR_SWAP_LIMITS, v)),
-            ATTR_ENCODER_READ: Attribute(None, default_type=np.int64, set_function=self.set_encoder_value),
-            ATTR_PULSE_READ: Attribute(None, default_type=np.int64, set_function=self.set_pulse_value, unit="pulse"),
-            ATTR_POSITION: Attribute(None, default_type=np.int64, set_function=self._move_absolute, unit="nm",
-                                     set_value_when_set_function=False),
-            ATTR_REFERENCING_METHOD: Attribute(None, default_type=np.uint8, min_value=1, max_value=15,
-                                               set_function=lambda v: self.set_sys(ATTR_REFERENCING_METHOD, v)),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.int64, min_value=-134217728,
-                                                 max_value=134217727, unit="nm",
-                                                 set_function=lambda v: self.set_sys(ATTR_POSITION_LOWER_LIMIT, v)),
-            ATTR_STEP_RESOLUTION: Attribute(500, default_type=np.int64, min_value=1,
-                                            max_value=134217727, unit="nm"),
-            ATTR_START_SPEED: Attribute(1, default_type=np.int32, min_value=1, max_value=2.5e6, unit="pulse/s",
-                                        set_function=lambda v: self.write_speed_table(start_speed=v)),
-            ATTR_TOP_SPEED: Attribute(1, default_type=np.int32, min_value=2, max_value=5e6, unit="pulse/s",
-                                      set_function=lambda v: self.write_speed_table(top_speed=v)),
-            ATTR_ACCELERATION_TIME: Attribute(1, default_type=np.int32, min_value=1, max_value=10e3, unit="ms",
-                                              set_function=lambda v: self.write_speed_table(acceleration_time=v)),
-            ATTR_DECELERATION_TIME: Attribute(1, default_type=np.int32, min_value=1, max_value=10e3, unit="ms",
-                                              set_function=lambda v: self.write_speed_table(deceleration_time=v)),
-            ATTR_ENCODER_RESOLUTION: Attribute(50, default_type=np.int64, min_value=1,
-                                               max_value=134217727, unit="nm"),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.int64, min_value=-134217728,
-                                                 max_value=134217727, unit="nm",
-                                                 set_function=lambda v: self.set_sys(ATTR_POSITION_UPPER_LIMIT, v)),
-            ATTR_MAXIMUM_VELOCITY: Attribute(None, default_type=np.uint32, min_value=2, max_value=5000000,
-                                             set_function=lambda v: self.set_sys(ATTR_MAXIMUM_VELOCITY, v)),
-            ATTR_ENCODER_CALC_NUM: Attribute(None, default_type=np.uint32, min_value=1, max_value=134217727,
-                                             set_function=lambda v: self.set_sys(ATTR_ENCODER_CALC_NUM, v)),
-            ATTR_ENCODER_CALC_DEN: Attribute(None, default_type=np.uint32, min_value=1, max_value=134217727,
-                                             set_function=lambda v: self.set_sys(ATTR_ENCODER_CALC_DEN, v)),
-            ATTR_FEEDBACK_CONTROL_TYPE: Attribute(None, default_type=DeviceKohzu.feedback_control_method.keys(),
-                                                  set_function=lambda v: self.set_sys(ATTR_FEEDBACK_CONTROL_TYPE,
-                                                                                      DeviceKohzu.feedback_control_method[
-                                                                                          v])),
-            ATTR_RETRY_COUNT: Attribute(None, default_type=np.uint16, min_value=1, max_value=10e3,
-                                        set_function=lambda v: self.set_sys(ATTR_RETRY_COUNT, v)),
-            ATTR_FEEDBACK_TIMEOUT: Attribute(None, default_type=np.uint16, min_value=1, max_value=10e3, unit="ms",
-                                             set_function=lambda v: self.set_sys(ATTR_FEEDBACK_TIMEOUT, v)),
-            ATTR_TRIGGER_SOURCE: Attribute(None, default_type=DeviceKohzu.trigger_source.keys(),
-                                           set_function=lambda v: self.set_sys(ATTR_TRIGGER_SOURCE,
-                                                                               DeviceKohzu.trigger_source[v])),
-            ATTR_TRIGGER_EDGE: Attribute(None, default_type=DeviceKohzu.trigger_edge.keys(),
-                                         set_function=lambda v: self.set_sys(ATTR_TRIGGER_EDGE,
-                                                                             DeviceKohzu.trigger_edge[v])),
-            ATTR_TRIGGER_PM_PITCH: Attribute(None, default_type=np.uint32, min_value=1, max_value=100e3,
-                                             set_function=lambda v: self.set_sys(ATTR_TRIGGER_PM_PITCH, v)),
-            ATTR_TRIGGER_ENC_PITCH: Attribute(None, default_type=np.uint32, min_value=1, max_value=100e3,
-                                              set_function=lambda v: self.set_sys(ATTR_TRIGGER_ENC_PITCH, v)),
-            ATTR_TRIGGER_PULSE_WIDTH: Attribute(None, default_type=DeviceKohzu.trigger_pulse_width.keys(),
-                                                set_function=lambda v: self.set_sys(ATTR_TRIGGER_PULSE_WIDTH,
-                                                                                    DeviceKohzu.trigger_pulse_width[
-                                                                                        v])),
-            ATTR_ENCODER_ADDING: Attribute(None, default_type=DeviceKohzu.encoder_adding.keys(),
-                                           set_function=lambda v: self.set_sys(ATTR_ENCODER_ADDING,
-                                                                               DeviceKohzu.encoder_adding[
-                                                                                   v])),
-            ATTR_TRIGGER_LOGIC: Attribute(None, default_type=DeviceKohzu.trigger_logic.keys(),
-                                          set_function=lambda v: self.set_sys(ATTR_TRIGGER_LOGIC,
-                                                                              DeviceKohzu.trigger_logic[
-                                                                                  v])),
-            ATTR_MOTOR_EXCITATION: Attribute(None, default_type=DeviceKohzu.motor_excitation.keys(),
-                                             set_function=lambda v: self.set_sys(ATTR_MOTOR_EXCITATION,
-                                                                                 DeviceKohzu.motor_excitation[
-                                                                                     v])),
-            ATTR_MOTOR_SELECTION: Attribute(None, default_type=DeviceKohzu.motor_selection.keys(),
-                                            set_function=lambda v: self.set_sys(ATTR_MOTOR_SELECTION,
-                                                                                DeviceKohzu.motor_selection[
-                                                                                    v])),
-            ATTR_MICROSTEP_SELECT: Attribute(None, default_type=DeviceKohzu.microstep_select.keys(),
-                                             set_function=lambda v: self.set_sys(ATTR_MICROSTEP_SELECT,
-                                                                                 DeviceKohzu.microstep_select[
-                                                                                     v])),
-            ATTR_STOP_TYPE: Attribute(None, default_type=DeviceKohzu.stop_type.keys(),
-                                      set_function=lambda v: self.set_sys(ATTR_STOP_TYPE,
-                                                                          DeviceKohzu.stop_type[
-                                                                              v])),
-            ATTR_ACCELERATION_PATTERN: Attribute(None, default_type=DeviceKohzu.accelerating_pattern.keys(),
-                                                 set_function=lambda v: self.write_speed_table(
-                                                     acceleration_pattern=v)),
-            ATTR_READ_ENCODER: Attribute(True, readonly=False, default_type=np.bool),
-            ATTR_VELOCITY: Attribute(0, default_type=np.int64, min_value=0, max_value=3e6, unit="nm/s",
-                                     set_function=self.set_velocity),
-        })
-
     def set_velocity(self, value):
         value = int(value / self[ATTR_STEP_RESOLUTION][VALUE])
-        self.set_attribute([ATTR_START_SPEED, VALUE], 2)
-        self.set_attribute([ATTR_TOP_SPEED, VALUE], 10)
-        self.set_attribute([ATTR_TOP_SPEED, VALUE], value)
-        self.set_attribute([ATTR_START_SPEED, VALUE], int(value / 2.2))
+        self.set_value(ATTR_START_SPEED, 2)
+        self.set_value(ATTR_TOP_SPEED, 10)
+        self.set_value(ATTR_TOP_SPEED, value)
+        self.set_value(ATTR_START_SPEED, int(value / 2.2))
 
     def subject_update(self, key, value, subject):
         super(DeviceKohzuChannel, self).subject_update(key, value, subject)
@@ -408,9 +392,9 @@ class DeviceKohzuChannel(DeviceChannel):
             elif attribute == ATTR_STOP_TYPE:
                 value = DeviceKohzu.stop_type.inv[value]
             elif attribute == ATTR_ENCODER_READ and self.get_value(ATTR_READ_ENCODER):
-                self.set_value(ATTR_POSITION, value * self[ATTR_ENCODER_RESOLUTION][VALUE])
+                self.set_raw_value(ATTR_POSITION, value * self[ATTR_ENCODER_RESOLUTION][VALUE])
             elif attribute == ATTR_PULSE_READ and not self.get_value(ATTR_READ_ENCODER):
-                self.set_value(ATTR_POSITION, value * self[ATTR_STEP_RESOLUTION][VALUE])
+                self.set_raw_value(ATTR_POSITION, value * self[ATTR_STEP_RESOLUTION][VALUE])
             elif attribute == ATTR_POSITION_LOWER_LIMIT:
                 value = self[ATTR_POSITION].remove_offset_factor(value)
                 self.set_attribute([ATTR_POSITION, MIN], value)
@@ -454,8 +438,8 @@ class DeviceKohzuChannel(DeviceChannel):
 
                 value = int(value[0])
 
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
     def start_polling(self):
         super(DeviceKohzuChannel, self).start_polling()
@@ -533,13 +517,13 @@ class DeviceKohzuChannel(DeviceChannel):
         rotational_direction = not self[ATTR_ROTATION_CHANGE][VALUE]
         swap_limits = not self[ATTR_SWAP_LIMITS][VALUE]
 
-        self.set_attribute([ATTR_ROTATION_CHANGE, VALUE], rotational_direction)
-        self.set_attribute([ATTR_SWAP_LIMITS, VALUE], swap_limits)
+        self.set_value(ATTR_ROTATION_CHANGE, rotational_direction)
+        self.set_value(ATTR_SWAP_LIMITS, swap_limits)
 
         def _swap_rotation_back(*_, **__):
             self.start_polling()
-            self.set_attribute([ATTR_ROTATION_CHANGE, VALUE], int(not rotational_direction))
-            self.set_attribute([ATTR_SWAP_LIMITS, VALUE], int(not swap_limits))
+            self.set_value(ATTR_ROTATION_CHANGE, int(not rotational_direction))
+            self.set_value(ATTR_SWAP_LIMITS, int(not swap_limits))
             if callback is not None:
                 callback()
 
@@ -566,7 +550,7 @@ class DeviceKohzuChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move to absolute position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), position, callback=callback)
+            return self.set_value(ATTR_POSITION, position, callback=callback)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -576,7 +560,7 @@ class DeviceKohzuChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move by relative position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), self[ATTR_POSITION].value() + position, callback=callback)
+            return self.set_value(ATTR_POSITION, self[ATTR_POSITION].value() + position, callback=callback)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
