@@ -7,7 +7,6 @@ from pint.errors import UndefinedUnitError
 
 from kamzik3 import DeviceError, DeviceUnitError, units
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.deviceSocket import DeviceSocket
 from kamzik3.snippets.snippetsDecorators import expose_method
@@ -132,6 +131,12 @@ class DeviceLexium(DeviceSocket):
     def __init__(self, host, port=503, device_id=None, config=None):
         super().__init__(host, port, device_id, config)
 
+    def _init_attributes(self):
+        super(DeviceLexium, self)._init_attributes()
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_TEMPERATURE, readonly=True, default_type=np.float16, decimals=2, unit="degC")
+        self.create_attribute(ATTR_CHANNELS, default_value=1, readonly=True)
+
     def handle_configuration(self):
 
         start_at = time.time()
@@ -146,15 +151,6 @@ class DeviceLexium(DeviceSocket):
         self.get_error_flag()
         self.get_serial_number(callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DeviceLexium, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_TEMPERATURE: Attribute(readonly=True, default_type=np.float16, decimals=2, unit="degC"),
-            ATTR_CHANNELS: Attribute(1, readonly=True)
-        })
-
     def handle_readout(self, readout_buffer):
         command, output, callback, token = super(DeviceLexium, self).handle_readout(readout_buffer)
         self.handle_command(command, output)
@@ -173,9 +169,9 @@ class DeviceLexium(DeviceSocket):
         command = command.strip()
         data = output.replace(command, "").strip()
         if command == "PR SN":
-            self.set_attribute((ATTR_SERIAL_NUMBER, VALUE), data)
+            self.set_raw_value(ATTR_SERIAL_NUMBER, data)
         elif command == "PR IT":
-            self.set_attribute((ATTR_TEMPERATURE, VALUE), float(data.replace(", ", ".")))
+            self.set_raw_value(ATTR_TEMPERATURE, float(data.replace(", ", ".")))
         elif command == "PR EF":
             if int(data) == 1:
                 self.handle_command_error(command, output)
@@ -265,50 +261,45 @@ class DeviceLexiumChannel(DeviceChannel):
 
     def _init_attributes(self):
         super(DeviceLexiumChannel, self)._init_attributes()
-        self.attributes.update({
-            ATTR_POSITION: Attribute(default_type=np.float64, readonly=False,
-                                     decimals=3,
-                                     set_function=self._move_absolute, set_value_when_set_function=False),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.float64, decimals=3,
-                                                 set_function=self.set_lower_position_limit),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.float64, decimals=3,
-                                                 set_function=self.set_upper_position_limit),
-            ATTR_BACKLASH_DISTANCE: Attribute(0, default_type=np.float64, readonly=False, decimals=3),
-            ATTR_STALL_FLAG: Attribute(default_type=np.bool, readonly=True),
-            ATTR_MOVING: Attribute(default_type=np.bool, readonly=True),
-            ATTR_VELOCITY: Attribute(default_type=np.uint64, readonly=True),
-            ATTR_HOLDING_CURRENT: Attribute(default_type=np.uint8, min_value=0, max_value=100, unit="%",
-                                            set_function=self.set_holding_current),
-            ATTR_INITIAL_VELOCITY: Attribute(default_type=np.uint64, unit="steps/s",
-                                             set_function=self.set_initial_velocity),
-            ATTR_MAXIMUM_VELOCITY: Attribute(default_type=np.uint64, unit="steps/s",
-                                             set_function=self.set_maximum_velocity),
-            ATTR_RUN_CURRENT: Attribute(default_type=np.uint8, max_value=100, unit="%",
-                                        set_function=self.set_run_current),
-            ATTR_ROTATION_DIRECTION: Attribute(default_type=np.uint8, max_value=1,
-                                               set_function=self.set_rotation_direction),
-            ATTR_TORQUE_SPEED: Attribute(default_type=np.uint64,
-                                         set_function=self.set_torque_speed),
-            ATTR_ENCODER_ENABLED: Attribute(default_type=np.uint8, min_value=0, max_value=1,
-                                            set_function=self.set_encoder_enabled),
-            ATTR_MICROSTEP_RESOLUTION: Attribute("None", default_type=self.available_resolutions,
-                                                 set_function=self.set_microstep_resolution, unit="ustep/step"),
-            ATTR_ENCODER_RESOLUTION: Attribute(4000, default_type=np.uint64,
-                                               unit="count/rev"),
-            ATTR_STEPS_PER_REVOLUTION: Attribute(200, default_type=np.uint64,
-                                                 unit="steps"),
-            ATTR_STALL_FACTOR: Attribute(default_type=np.uint64, min_value=0, max_value=65000,
-                                         unit="counts", set_function=self.set_stall_factor),
-            ATTR_POSITIVE_LIMIT_SWITCH: Attribute(default_type=np.bool, readonly=True),
-            ATTR_NEGATIVE_LIMIT_SWITCH: Attribute(default_type=np.bool, readonly=True),
-            ATTR_INDEX_MARK: Attribute(default_type=np.bool, readonly=True),
-            ATTR_USE_POSITIVE_LIMIT_SWITCH: Attribute(default_type=np.bool,
-                                                      set_function=lambda value: self.set_digital_input(3, 2, value)),
-            ATTR_USE_NEGATIVE_LIMIT_SWITCH: Attribute(default_type=np.bool,
-                                                      set_function=lambda value: self.set_digital_input(4, 3, value)),
-            ATTR_REFERENCING: Attribute(default_type=np.bool, readonly=True),
-            ATTR_USE_MARK_INDEX: Attribute(True, default_type=np.bool, readonly=False),
-        })
+        self.create_attribute(ATTR_POSITION, default_type=np.float64, readonly=False, decimals=3,
+                              set_function=self._move_absolute, set_value_when_set_function=False)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.float64, decimals=3,
+                              set_function=self.set_lower_position_limit)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.float64, decimals=3,
+                              set_function=self.set_upper_position_limit)
+        self.create_attribute(ATTR_BACKLASH_DISTANCE, default_value=0, default_type=np.float64, readonly=False,
+                              decimals=3)
+        self.create_attribute(ATTR_STALL_FLAG, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_MOVING, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.uint64, readonly=True)
+        self.create_attribute(ATTR_HOLDING_CURRENT, default_type=np.uint8, min_value=0, max_value=100, unit="%",
+                              set_function=self.set_holding_current)
+        self.create_attribute(ATTR_INITIAL_VELOCITY, default_type=np.uint64, unit="steps/s",
+                              set_function=self.set_initial_velocity)
+        self.create_attribute(ATTR_MAXIMUM_VELOCITY, default_type=np.uint64, unit="steps/s",
+                              set_function=self.set_maximum_velocity)
+        self.create_attribute(ATTR_RUN_CURRENT, default_type=np.uint8, max_value=100, unit="%",
+                              set_function=self.set_run_current)
+        self.create_attribute(ATTR_ROTATION_DIRECTION, default_type=np.uint8, max_value=1,
+                              set_function=self.set_rotation_direction)
+        self.create_attribute(ATTR_TORQUE_SPEED, default_type=np.uint64, set_function=self.set_torque_speed)
+        self.create_attribute(ATTR_ENCODER_ENABLED, default_type=np.uint8, min_value=0, max_value=1,
+                              set_function=self.set_encoder_enabled)
+        self.create_attribute(ATTR_MICROSTEP_RESOLUTION, default_value="None", default_type=self.available_resolutions,
+                              set_function=self.set_microstep_resolution, unit="ustep/step")
+        self.create_attribute(ATTR_ENCODER_RESOLUTION, default_value=4000, default_type=np.uint64, unit="count/rev")
+        self.create_attribute(ATTR_STEPS_PER_REVOLUTION, default_value=200, default_type=np.uint64, unit="steps")
+        self.create_attribute(ATTR_STALL_FACTOR, default_type=np.uint64, min_value=0, max_value=65000,
+                              unit="counts", set_function=self.set_stall_factor)
+        self.create_attribute(ATTR_POSITIVE_LIMIT_SWITCH, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_NEGATIVE_LIMIT_SWITCH, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_INDEX_MARK, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_USE_POSITIVE_LIMIT_SWITCH, default_type=np.bool,
+                              set_function=lambda value: self.set_digital_input(3, 2, value))
+        self.create_attribute(ATTR_USE_NEGATIVE_LIMIT_SWITCH, default_type=np.bool,
+                              set_function=lambda value: self.set_digital_input(4, 3, value))
+        self.create_attribute(ATTR_REFERENCING, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_USE_MARK_INDEX, default_value=True, default_type=np.bool, readonly=False)
 
     def handle_configuration(self):
         if self.configured:
@@ -380,7 +371,7 @@ class DeviceLexiumChannel(DeviceChannel):
             elif key[1] == ATTR_STALL_FLAG and value is True:
                 self.command(u"ST=0\r")
 
-            self.set_value(key[1], value)
+            self.set_raw_value(key[1], value)
 
     @expose_method()
     def stop(self, callback=None):
@@ -446,7 +437,7 @@ class DeviceLexiumChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move motor channel to absolute position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), float(position.m), callback)
+            return self.set_value(ATTR_POSITION, float(position.m), callback)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -462,11 +453,9 @@ class DeviceLexiumChannel(DeviceChannel):
             raise DeviceUnitError(e)
 
     def set_lower_position_limit(self, value, callback=None):
-        self.set_value(ATTR_POSITION_LOWER_LIMIT, value)
         return self.set_attribute([ATTR_POSITION, MIN], value, callback)
 
     def set_upper_position_limit(self, value, callback=None):
-        self.set_value(ATTR_POSITION_UPPER_LIMIT, value)
         return self.set_attribute([ATTR_POSITION, MAX], value, callback)
 
     @expose_method({"position": ATTR_POSITION})
