@@ -1,14 +1,14 @@
 import re
 import time
+from decimal import DivisionByZero
+from math import ceil
 
 import bidict
 import numpy as np
 import serial
-from decimal import DivisionByZero
-from pint.errors import UndefinedUnitError
-from math import ceil
-from numpy import around, sign, sqrt
 from bidict import frozenbidict, MutableBidict
+from numpy import around, sign, sqrt
+from pint.errors import UndefinedUnitError
 
 from kamzik3 import DeviceUnitError, DeviceError
 from kamzik3.constants import *
@@ -74,6 +74,7 @@ REC_OPEN_LOOP = 14
 REC_CONTROL_OUTPUT = 15
 REC_SLOWED_TARGET = 22
 
+
 class DevicePi(DeviceSocket):
     terminator = b"\n"
     error_pattern = re.compile('^([-0-9]+)$')
@@ -286,20 +287,16 @@ class DevicePi(DeviceSocket):
         self.channel_mapping = MutableBidict()
         self.valid_commands = []
         super().__init__(host, port, device_id, config)
-        self._expose_methods_to_clients()
 
     def _init_attributes(self):
         super(DevicePi, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_CHANNELS: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_SYNTAX_VERSION: Attribute(None, readonly=True, default_type=np.float16, decimals=3),
-            ATTR_POSITIONS: Attribute(None, readonly=True),
-            ATTR_TRIGGER_OUTPUTS_COUNT:  Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_WAVE_GEN_COUNT: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_SERVO_UPDATE_TIME: Attribute(None, readonly=True, default_type=np.float64, decimals=12, unit="ms"),
-        })
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_CHANNELS, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_SYNTAX_VERSION, readonly=True, default_type=np.float16, decimals=3)
+        self.create_attribute(ATTR_POSITIONS, readonly=True)
+        self.create_attribute(ATTR_TRIGGER_OUTPUTS_COUNT, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_WAVE_GEN_COUNT, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_SERVO_UPDATE_TIME, readonly=True, default_type=np.float64, decimals=12, unit="ms")
 
     def valid_command_format(self, command):
         if command is not None and command[-1] == "\n":
@@ -346,19 +343,19 @@ class DevicePi(DeviceSocket):
                     self.attributes[trigger_group_name] = {}
                     trigger_output_count += 1
 
-                self.attributes[trigger_group_name][ATTR_TRIGGER_LINE_DATA] = Attribute(None, readonly=True)
+                self.create_attribute(ATTR_TRIGGER_LINE_DATA, trigger_group_name, readonly=True)
                 set_attribute = lambda value, parameter=attribute, trigger_output=trigger_output: self.set_configuration_of_trigger_output(trigger_output, parameter, value)
                 if attribute == TRIG_OUTPUT_AXIS:
                     set_attribute = lambda value, parameter=attribute, trigger_output=trigger_output: self.set_configuration_of_trigger_output(trigger_output, parameter, int(value) + 1)
-                    self.attributes[trigger_group_name][attribute] = Attribute(int(value) - 1, default_type=np.uint16,
+                    self.create_attribute(attribute, trigger_group_name, default_value=int(value) - 1, default_type=np.uint16,
                                                                                set_function=set_attribute)
                 elif attribute == TRIG_OUTPUT_MODE:
-                    self.attributes[trigger_group_name][attribute] = Attribute(trigger_output_mode.inv.get(int(value)),
+                    self.create_attribute(attribute, trigger_group_name, default_value=trigger_output_mode.inv.get(int(value)),
                                                                                default_type=list(
                                                                                    trigger_output_mode.keys()),
                                                                                set_function=set_attribute)
                 else:
-                    self.attributes[trigger_group_name][attribute] = Attribute(float(value), default_type=np.float64,
+                    self.create_attribute(attribute, trigger_group_name, default_value=float(value), default_type=np.float64,
                                                                                decimals=12, set_function=set_attribute)
 
         self.set_value(ATTR_TRIGGER_OUTPUTS_COUNT, trigger_output_count)
@@ -370,25 +367,22 @@ class DevicePi(DeviceSocket):
             generator_id = int(generator_id) - 1
             mode = int(mode)
             generator_group_name = "Wave generator {}".format(generator_id)
-            self.attributes[generator_group_name] = {}
-            self.attributes[generator_group_name][ATTR_STATUS] = Attribute(STATUS_IDLE, readonly=True)
-            self.attributes[generator_group_name][ATTR_WAVETABLE_DATA] = Attribute(None, readonly=True)
-            self.attributes[generator_group_name][ATTR_RECORD_TABLE_DATA] = Attribute(None, readonly=True)
-            self.attributes[generator_group_name][ATTR_WAVETABLE_LENGTH] = Attribute(None, readonly=True,
-                                                                                     default_type=np.uint32)
+            self.create_attribute(ATTR_STATUS, generator_group_name, default_value=STATUS_IDLE, readonly=True)
+            self.create_attribute(ATTR_WAVETABLE_DATA, generator_group_name, readonly=True)
+            self.create_attribute(ATTR_RECORD_TABLE_DATA, generator_group_name, readonly=True)
+            self.create_attribute(ATTR_WAVETABLE_LENGTH, generator_group_name, readonly=True, default_type=np.uint32)
             set_function = lambda offset, gen_id=generator_id: self.set_wave_gen_output_offset(gen_id, offset)
-            self.attributes[generator_group_name][ATTR_WAVETABLE_OFFSET] = Attribute(None, readonly=False,
-                                                                                     default_type=np.float64, decimals=12, set_function=set_function)
-            self.attributes[generator_group_name][ATTR_WAVE_TABLE] = Attribute(None, readonly=True,
-                                                                                     default_type=np.uint32)
-
-            set_function = lambda table_rate, gen_id=generator_id: self.set_wave_generator_rate_and_interpolation_type(gen_id, wave_table_rate=table_rate)
-            self.attributes[generator_group_name][ATTR_WAVETABLE_RATE] = Attribute(None, default_type=np.uint32, set_function=set_function)
+            self.create_attribute(ATTR_WAVETABLE_OFFSET, generator_group_name, readonly=False, decimals=12,
+                                                                                     default_type=np.float64,
+                                                                                     set_function=set_function)
+            self.create_attribute(ATTR_WAVE_TABLE, generator_group_name, readonly=True, default_type=np.uint32)
 
+            set_function = lambda table_rate, gen_id=generator_id: self.set_wave_generator_rate_and_interpolation_type(
+                gen_id, wave_table_rate=table_rate)
+            self.create_attribute(ATTR_WAVETABLE_RATE, generator_group_name, default_type=np.uint32, set_function=set_function)
 
             set_function = lambda interpolation_type, gen_id=generator_id: self.set_wave_generator_rate_and_interpolation_type(gen_id, interpolation_type=interpolation_type)
-            self.attributes[generator_group_name][ATTR_WAVETABLE_INTERPOLATION] = Attribute(None, default_type=list(
-                                                                                   data_interpolation.keys()), set_function=set_function)
+            self.create_attribute(ATTR_WAVETABLE_INTERPOLATION, generator_group_name, default_type=list(data_interpolation.keys()), set_function=set_function)
 
         self.set_value(ATTR_WAVE_GEN_COUNT, len(output))
         self.get_wave_generator_rate_and_interpolation_type()
@@ -429,16 +423,16 @@ class DevicePi(DeviceSocket):
                 self.handle_command_error(command, readout_buffer)
 
         if command == "IDN?\n":
-            self.set_value(ATTR_SERIAL_NUMBER, "{}".format(output))
+            self.set_raw_value(ATTR_SERIAL_NUMBER, "{}".format(output))
         elif command == "SAI?\n":
             for channel_id in output.split(" \n"):
                 try:
                     self.channel_mapping[channel_id] = len(self.channel_mapping)
                 except bidict._exc.KeyAndValueDuplicationError:
                     continue
-            self.set_value(ATTR_CHANNELS, len(self.channel_mapping))
+            self.set_raw_value(ATTR_CHANNELS, len(self.channel_mapping))
         elif command == "CSV?\n":
-            self.set_value(ATTR_SYNTAX_VERSION, float(output))
+            self.set_raw_value(ATTR_SYNTAX_VERSION, float(output))
         elif command == "HLP?\n":
             for valid_command in output.split("\n")[1:-1]:
                 cmd, desc = valid_command.split(" - ")
@@ -453,7 +447,8 @@ class DevicePi(DeviceSocket):
                 state = bool(stat & (1 << generator_id))
                 if not state and self.get_value(("Wave generator {}".format(generator_id), ATTR_STATUS)) == STATUS_BUSY:
                     self.get_recorder_data(generator_id)
-                self.set_value(("Wave generator {}".format(generator_id), ATTR_STATUS), STATUS_BUSY if state else STATUS_IDLE)
+                self.set_raw_value(("Wave generator {}".format(generator_id), ATTR_STATUS),
+                               STATUS_BUSY if state else STATUS_IDLE)
                 trigger_axis = self.get_value(("Trigger output {}".format(generator_id), TRIG_OUTPUT_AXIS))
                 self.notify((trigger_axis, ATTR_SCANNING), state)
             if stat == 0:
@@ -470,7 +465,7 @@ class DevicePi(DeviceSocket):
                 channel, position = position.split("=")
                 positions_list.append(position)
                 self.notify((self.channel_mapping[channel], ATTR_POSITION), float(position))
-            self.set_value(ATTR_POSITIONS, " ".join(positions_list))
+            self.set_raw_value(ATTR_POSITIONS, " ".join(positions_list))
         elif "FRF?" in command:
             channel, flag = output.split("=")
             self.notify((self.channel_mapping[channel], ATTR_REFERENCED), bool(int(flag)))
@@ -490,12 +485,13 @@ class DevicePi(DeviceSocket):
             lines = output.split(" \n")
             for line in lines:
                 generator_id, generator_offset = line.split("=")
-                self.set_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_OFFSET], float(generator_offset))
+                self.set_raw_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_OFFSET],
+                               float(generator_offset))
         elif "WSL?" in command:
             lines = output.split(" \n")
             for line in lines:
                 generator_id, wave_table_id = line.split("=")
-                self.set_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVE_TABLE], int(wave_table_id))
+                self.set_raw_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVE_TABLE], int(wave_table_id))
         elif "WTR?" in command:
             lines = output.split(" \n")
             for line in lines:
@@ -504,14 +500,14 @@ class DevicePi(DeviceSocket):
                 interpolation = data_interpolation.inv.get(int(interpolation))
                 if int(generator_id) == 0:
                     for generator_id in range(self.get_value(ATTR_WAVE_GEN_COUNT)):
-                        self.set_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_RATE],
+                        self.set_raw_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_RATE],
                                        int(rate))
-                        self.set_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_INTERPOLATION],
+                        self.set_raw_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_INTERPOLATION],
                                        interpolation)
                 else:
-                    self.set_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_RATE],
+                    self.set_raw_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_RATE],
                                    int(rate))
-                    self.set_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_INTERPOLATION],
+                    self.set_raw_value(["Wave generator {}".format(int(generator_id) - 1), ATTR_WAVETABLE_INTERPOLATION],
                                    interpolation)
         elif "SEP?" in command:
             lines = output.split(" \n")
@@ -520,26 +516,26 @@ class DevicePi(DeviceSocket):
                 _, parameter = parameter.split(" ")
 
                 if parameter == SERVO_UPDATE_TIME_PARAM:
-                    self.set_value(ATTR_SERVO_UPDATE_TIME, float(value) * 1e3)
+                    self.set_raw_value(ATTR_SERVO_UPDATE_TIME, float(value) * 1e3)
         elif "GWD?" in command:
             lines = output.split(" \n")
             wave_table_id = int(lines[5][-1])
             wave_table_data = ", ".join(lines[7:])
             generator_id = wave_table_id - 1
-            self.set_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_DATA],
+            self.set_raw_value(["Wave generator {}".format(generator_id), ATTR_WAVETABLE_DATA],
                            wave_table_data)
         elif "TWS?" in command:
             lines = output.split(" \n")
             cmd, start, end, trigger_output = command.split(" ")
             trigger_group_name = "Trigger output {}".format(int(trigger_output) - 1)
             triggers_level = ", ".join(lines[6:])
-            self.set_value([trigger_group_name, ATTR_TRIGGER_LINE_DATA], triggers_level)
+            self.set_raw_value([trigger_group_name, ATTR_TRIGGER_LINE_DATA], triggers_level)
         elif "DRR?" in command:
             lines = output.split(" \n")
             wave_table_id = int(lines[5][-1])
             recorded_data = ", ".join(lines[7:])
             generator_id = wave_table_id - 1
-            self.set_value(["Wave generator {}".format(generator_id), ATTR_RECORD_TABLE_DATA],
+            self.set_raw_value(["Wave generator {}".format(generator_id), ATTR_RECORD_TABLE_DATA],
                            recorded_data)
 
             trigger_group_name = "Trigger output {}".format(generator_id)
@@ -561,7 +557,8 @@ class DevicePi(DeviceSocket):
 
     def get_memory_parameters(self, list_of_parameters, callback=None):
         assert isinstance(list_of_parameters, list)
-        attributes = " ".join([" ".join(c) for c in zip([str(b + 1) for b in range(len(list_of_parameters))], list_of_parameters)])
+        attributes = " ".join(
+            [" ".join(c) for c in zip([str(b + 1) for b in range(len(list_of_parameters))], list_of_parameters)])
         return self.command("SEP? {}\n".format(attributes), with_token=True, callback=callback)
 
     def clear_wave_table(self, wave_table_id):
@@ -596,7 +593,8 @@ class DevicePi(DeviceSocket):
         """
         return self.command("WTR?\n", with_token=True, callback=callback)
 
-    def set_wave_generator_rate_and_interpolation_type(self, wave_generator_id=0, wave_table_rate=None, interpolation_type=None,
+    def set_wave_generator_rate_and_interpolation_type(self, wave_generator_id=0, wave_table_rate=None,
+                                                       interpolation_type=None,
                                                        callback=None):
         """
         Set wave generator interpolation type.
@@ -610,11 +608,12 @@ class DevicePi(DeviceSocket):
         if wave_table_rate is None:
             wave_table_rate = self.get_value(["Wave generator {}".format(wave_generator_id), ATTR_WAVETABLE_RATE])
         if interpolation_type is None:
-            interpolation_type = self.get_value(["Wave generator {}".format(wave_generator_id), ATTR_WAVETABLE_INTERPOLATION])
+            interpolation_type = self.get_value(
+                ["Wave generator {}".format(wave_generator_id), ATTR_WAVETABLE_INTERPOLATION])
         interpolation_type = data_interpolation.get(interpolation_type)
         self.logger.info("Set wave generator interpolation type to {}".format(wave_generator_id))
         output = self.command("WTR {} {} {}\n".format(wave_generator_id, int(wave_table_rate), interpolation_type),
-                            returning=False, with_token=True, callback=callback)
+                              returning=False, with_token=True, callback=callback)
         self.get_wave_generator_rate_and_interpolation_type()
         return output
 
@@ -677,7 +676,8 @@ class DevicePi(DeviceSocket):
 
     def set_connected_table_to_generator(self, wave_generator_id, wave_table_id, callback=None):
         self.logger.info("Set connected generator {} to table {}".format(wave_generator_id, wave_table_id))
-        return self.command("WSL {} {}\n".format(wave_generator_id + 1, wave_table_id + 1), returning=False, with_token=True,
+        return self.command("WSL {} {}\n".format(wave_generator_id + 1, wave_table_id + 1), returning=False,
+                            with_token=True,
                             callback=callback)
 
     def get_configuration_of_trigger_outputs(self, callback=None):
@@ -727,12 +727,14 @@ class DevicePi(DeviceSocket):
     def get_trigger_line(self, trigger_output, start_point=1, number_of_points=None, callback=None):
         if number_of_points is None:
             number_of_points = self.get_value(["Wave generator {}".format(trigger_output), ATTR_WAVETABLE_LENGTH])
-        return self.command("TWS? {} {} {}\n".format(start_point, number_of_points, trigger_output + 1), with_token=True, callback=callback)
+        return self.command("TWS? {} {} {}\n".format(start_point, number_of_points, trigger_output + 1),
+                            with_token=True, callback=callback)
 
     def get_wavetable_data(self, wave_table_id, start_point=1, number_of_points=None, callback=None):
         if number_of_points is None:
             number_of_points = self.get_value(["Wave generator {}".format(wave_table_id), ATTR_WAVETABLE_LENGTH])
-        return self.command("GWD? {} {} {}\n".format(start_point, number_of_points, wave_table_id + 1), with_token=True, callback=callback)
+        return self.command("GWD? {} {} {}\n".format(start_point, number_of_points, wave_table_id + 1), with_token=True,
+                            callback=callback)
 
     def set_wave_form_definition(self, wave_table_id, wave_add_method, wave_type, wave_type_param, callback=None):
         if wave_type == WAVE_TYPE_PNT:
@@ -760,7 +762,8 @@ class DevicePi(DeviceSocket):
                 sliced += self.N_WAV_POINTS_PER_CMD
 
             if prepared_command is not None:
-                return self.command(_make_command(prepared_command, command_parameter, sliced_points), returning=False, with_token=True, callback=callback)
+                return self.command(_make_command(prepared_command, command_parameter, sliced_points), returning=False,
+                                    with_token=True, callback=callback)
 
         elif wave_type == WAVE_TYPE_SINP:
             """
@@ -779,7 +782,8 @@ class DevicePi(DeviceSocket):
             command_parameter = "%i %f %f %i %i %i" % wave_type_param
 
         command = "WAV %s %s %s" % (wave_table_id + 1, wave_add_method, wave_type)
-        return self.command("%s %s\n" % (command, command_parameter), returning=False, with_token=True, callback=callback)
+        return self.command("%s %s\n" % (command, command_parameter), returning=False, with_token=True,
+                            callback=callback)
 
     def set_record_table_rate(self, rate, callback=None):
         """
@@ -804,12 +808,14 @@ class DevicePi(DeviceSocket):
         :param callback:
         :return:
         """
-        return self.command("DRC {} {} {}\n".format(table_id + 1, source + 1, option), returning=False, with_token=True, callback=callback)
+        return self.command("DRC {} {} {}\n".format(table_id + 1, source + 1, option), returning=False, with_token=True,
+                            callback=callback)
 
     def get_recorder_data(self, table_id, start_point=1, number_of_points=None, callback=None):
         if number_of_points is None:
             number_of_points = self.get_value(["Wave generator {}".format(table_id), ATTR_WAVETABLE_LENGTH])
-        return self.command("DRR? {} {} {}\n".format(start_point, number_of_points, table_id + 1), with_token=True, callback=callback)
+        return self.command("DRR? {} {} {}\n".format(start_point, number_of_points, table_id + 1), with_token=True,
+                            callback=callback)
 
     def set_recorded_trigger_source(self, table_id, source, value, callback=None):
         """
@@ -822,7 +828,8 @@ class DevicePi(DeviceSocket):
         :param callback:
         :return:
         """
-        return self.command("DRT {} {} {}\n".format(table_id + 1, source, value), returning=False, with_token=True, callback=callback)
+        return self.command("DRT {} {} {}\n".format(table_id + 1, source, value), returning=False, with_token=True,
+                            callback=callback)
 
     def _move_absolute(self, channel, position, callback=None):
         self.logger.info(u"Move channel {} to absolute position {}".format(channel, position))
@@ -834,8 +841,10 @@ class DevicePi(DeviceSocket):
         self.set_wave_generator_mode(int(generator_id), 1)
         self.command("#9\n")
 
-    @expose_method({"channel": "0", "center": "Center of scan", "step_size": "steps size", "pixels": "Scan points", "dwell_time": "usec", "trigger_mode": "0", "interpolating": "0"})
-    def set_fly_scan(self, channel=0, center=50, step_size=1, pixels=10, dwell_time=100, trigger_mode=TRIG_MODE_GENERATOR, interpolating=False):
+    @expose_method({"channel": "0", "center": "Center of scan", "step_size": "steps size", "pixels": "Scan points",
+                    "dwell_time": "usec", "trigger_mode": "0", "interpolating": "0"})
+    def set_fly_scan(self, channel=0, center=50, step_size=1, pixels=10, dwell_time=100,
+                     trigger_mode=TRIG_MODE_GENERATOR, interpolating=False):
         self.stop_polling()
         self.set_status(STATUS_BUSY)
         channel = int(channel)
@@ -865,7 +874,8 @@ class DevicePi(DeviceSocket):
         # Set interpolation and table rate
         interpolation_type = INTERPOLATION_LINE if interpolating else INTERPOLATION_NONE
         wave_table_rate = dwell_time / self.get_value(ATTR_SERVO_UPDATE_TIME)
-        self.set_wave_generator_rate_and_interpolation_type(wave_table_rate=wave_table_rate, interpolation_type=interpolation_type)
+        self.set_wave_generator_rate_and_interpolation_type(wave_table_rate=wave_table_rate,
+                                                            interpolation_type=interpolation_type)
         # Set wave generator cycles
         self.set_wave_gen_cycles(generator_id, 1)
 
@@ -1042,6 +1052,23 @@ class DevicePiChannel(DeviceChannel):
         self.channel_id = None
         super().__init__(device, channel, device_id, config)
 
+    def _init_attributes(self):
+        super(DevicePiChannel, self)._init_attributes()
+        self.create_attribute(ATTR_ENCODER_BASE_UNIT, readonly=True)
+        self.create_attribute(ATTR_POSITION, default_type=np.float64, decimals=9,
+                                     set_function=self._move_absolute, set_value_when_set_function=False)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.float64, decimals=9,
+                                                 set_function=self.set_position_upper_limit)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.float64, decimals=9,
+                                                 set_function=self.set_position_lower_limit)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.float64, decimals=9, set_function=self.set_velocity)
+        self.create_attribute(ATTR_SERVO_STATE, default_type=np.bool, set_function=self.set_servo_state)
+        self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_MOVING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_SCANNING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_WAVE_GENERATOR, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_RECORD_TABLE_DATA, readonly=True)
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -1067,27 +1094,6 @@ class DevicePiChannel(DeviceChannel):
         self.get_position_upper_limit()
         self.get_velocity(callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DevicePiChannel, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_ENCODER_BASE_UNIT: Attribute(None, readonly=True),
-            ATTR_POSITION: Attribute(None, default_type=np.float64, decimals=9,
-                                     set_function=self._move_absolute, set_value_when_set_function=False),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.float64, decimals=9,
-                                                 set_function=self.set_position_upper_limit),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.float64, decimals=9,
-                                                 set_function=self.set_position_lower_limit),
-            ATTR_VELOCITY: Attribute(None, default_type=np.float64, decimals=9,
-                                     set_function=self.set_velocity),
-            ATTR_SERVO_STATE: Attribute(None, default_type=np.bool, set_function=self.set_servo_state),
-            ATTR_REFERENCED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_MOVING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_SCANNING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_WAVE_GENERATOR: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_RECORD_TABLE_DATA: Attribute(None, readonly=True),
-        })
-
     def subject_update(self, key, value, subject):
         super(DevicePiChannel, self).subject_update(key, value, subject)
 
@@ -1115,16 +1121,18 @@ class DevicePiChannel(DeviceChannel):
                 else:
                     self.set_status(STATUS_IDLE)
 
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
-    @expose_method({"Field_size": ATTR_POSITION, "Steps": "Number of steps", "Dwell_time": "milliseconds", "Interpolating": "bool"})
+    @expose_method({"Field_size": ATTR_POSITION, "Steps": "Number of steps", "Dwell_time": "milliseconds",
+                    "Interpolating": "bool"})
     def set_relative_fly_scan(self, Field_size, Steps, Dwell_time, Interpolating):
         filed_size = device_units(self, ATTR_POSITION, Field_size).m
         step_size = filed_size / int(Steps)
         actual_position = self.get_value(ATTR_POSITION)
-        generator_id = self.device.set_fly_scan(self.channel, actual_position, step_size, int(Steps), float(Dwell_time), interpolating=int(Interpolating))
-        self.set_attribute([ATTR_WAVE_GENERATOR, VALUE], generator_id)
+        generator_id = self.device.set_fly_scan(self.channel, actual_position, step_size, int(Steps), float(Dwell_time),
+                                                interpolating=int(Interpolating))
+        self.set_value(ATTR_WAVE_GENERATOR, generator_id)
         return generator_id
 
     @expose_method()
@@ -1175,7 +1183,7 @@ class DevicePiChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move motor channel to absolute position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), position.m)
+            return self.set_value(ATTR_POSITION, position.m)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -1185,7 +1193,7 @@ class DevicePiChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move by relative position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), self[ATTR_POSITION].value() + position)
+            return self.set_value(ATTR_POSITION, self[ATTR_POSITION].value() + position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
