@@ -4,18 +4,19 @@ import time
 from collections import OrderedDict
 
 import numpy as np
+import psutil
 
 import kamzik3
 from kamzik3 import DeviceUnknownError, DeviceError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.attributeLogger import AttributeLogger, ATTR_LOGGING
 from kamzik3.devices.device import Device
 from kamzik3.devices.deviceClient import DeviceClient
 from kamzik3.devices.general.measurementGroup import MeasurementGroup
 from kamzik3.snippets.snippetLogging import set_rotating_file_handler, set_sys_exception_handler, get_console_handler
 from kamzik3.snippets.snippetsControlLoops import control_asyncore_loop, control_device_poller, control_port_read_loop, \
-    control_device_client_poller
+    control_device_connection_poller
+from kamzik3.snippets.snippetsTimer import CallbackTimer
 
 
 class DeviceSession(Device):
@@ -30,26 +31,28 @@ class DeviceSession(Device):
         kamzik3.session = self
         self.devices = OrderedDict()
         self.measurement_groups = {}
+        self.polling_timer = CallbackTimer(1000, self.measure_cpu_ram)
+        self.os_process = psutil.Process(os.getpid())
         super(DeviceSession, self).__init__(device_id, config)
-        self._expose_methods_to_clients()
         self.connect()
 
     def _init_attributes(self):
         super(DeviceSession, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_LOG_DIRECTORY: Attribute(set_function=self.set_log_directory, readonly=True,
-                                          set_value_when_set_function=True),
-            ATTR_RESOURCE_DIRECTORY: Attribute(set_function=self.set_resource_directory, readonly=True,
-                                               set_value_when_set_function=True),
-            ATTR_DEVICES_COUNT: Attribute(0, default_type=np.uint64, readonly=True),
-            ATTR_ALLOW_CONSOLE_LOG: Attribute(True, default_type=np.bool),
-            ATTR_ALLOW_GUI_LOG: Attribute(True, default_type=np.bool),
-            ATTR_ALLOW_DEVICE_LOG: Attribute(True, default_type=np.bool),
-            ATTR_ALLOW_ATTRIBUTE_LOG: Attribute(True, default_type=np.bool),
-            ATTR_ALLOW_MACRO_LOG: Attribute(True, default_type=np.bool),
-            ATTR_LOOPS_RUNNING: Attribute(False, default_type=np.bool, readonly=True),
-        })
+        self.create_attribute(ATTR_LOG_DIRECTORY, set_function=self.set_log_directory, readonly=True)
+        self.create_attribute(ATTR_RESOURCE_DIRECTORY, set_function=self.set_resource_directory, readonly=True)
+        self.create_attribute(ATTR_DEVICES_COUNT, default_value=0, default_type=np.uint64, readonly=True)
+        self.create_attribute(ATTR_ALLOW_CONSOLE_LOG, default_value=True, default_type=np.bool)
+        self.create_attribute(ATTR_ALLOW_GUI_LOG, default_value=True, default_type=np.bool)
+        self.create_attribute(ATTR_ALLOW_DEVICE_LOG, default_value=True, default_type=np.bool)
+        self.create_attribute(ATTR_ALLOW_ATTRIBUTE_LOG, default_value=True, default_type=np.bool)
+        self.create_attribute(ATTR_ALLOW_MACRO_LOG, default_value=True, default_type=np.bool)
+        self.create_attribute(ATTR_LOOPS_RUNNING, default_value=False, default_type=np.bool, readonly=True)
+        self.create_attribute(ATTR_PROCESS_ID, default_value=os.getpid(), readonly=True)
+        self.create_attribute(ATTR_CPU_USAGE, default_value=0, default_type=np.float, readonly=True, unit="%", decimals=4)
+        self.create_attribute(ATTR_MEMORY_USAGE, default_value=0, default_type=np.float, readonly=True, decimals=3, unit="MB", factor=10e-7)
+        self.create_attribute(ATTR_AVAILABLE_MEMORY, default_value=0, default_type=np.float,
+                              readonly=True, unit="MB", factor=10e-7, decimals=3)
+        self.set_raw_value(ATTR_AVAILABLE_MEMORY, psutil.virtual_memory().total)
 
     def add_measurement_group(self, measurement_group):
         assert isinstance(measurement_group, MeasurementGroup)
@@ -60,8 +63,11 @@ class DeviceSession(Device):
         self.session = self
         self.set_value(ATTR_DEVICES_COUNT, len(self.devices))
 
-    def handle_configuration(self):
+    def measure_cpu_ram(self):
+        self.set_raw_value(ATTR_CPU_USAGE, self.os_process.cpu_percent())
+        self.set_raw_value(ATTR_MEMORY_USAGE, self.os_process.memory_info().rss)
 
+    def handle_configuration(self):
         start_at = time.time()
 
         def _finish_configuration(*_, **__):
@@ -80,6 +86,7 @@ class DeviceSession(Device):
             if self.get_value(ATTR_ALLOW_MACRO_LOG):
                 self.set_macro_log_directory(u"Macro")
 
+            self.start_polling()
             self.set_status(STATUS_CONFIGURED)
             self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))
 
@@ -96,6 +103,15 @@ class DeviceSession(Device):
                     logger.add_logged_attribute(device_id, attribute)
             logger.set_attribute((ATTR_LOGGING, VALUE), True)
 
+    def start_polling(self):
+        self.polling_timer.start()
+
+    def stop_polling(self):
+        self.polling_timer.stop()
+
+    def command(self, command, callback=None, with_token=False, returning=True):
+        raise DeviceError("Device does not accept any commands.")
+
     def get_device(self, device_id):
         try:
             return self.devices[device_id]
@@ -105,6 +121,11 @@ class DeviceSession(Device):
                 return DeviceClient(master_server, master_port, device_id)
             raise DeviceUnknownError("Device {} was not found.".format(device_id))
 
+    def change_device_id(self, device_id, new_device_id):
+        device = self.get_device(device_id)
+        device.device_id = new_device_id
+        self.devices[new_device_id] = self.devices.pop(device_id)
+
     def add_device(self, device):
         assert isinstance(device, Device)
 
@@ -120,7 +141,6 @@ class DeviceSession(Device):
 
     def remove_device(self, device):
         assert isinstance(device, Device)
-
         if device.device_id in self.devices:
             device.session = None
             del self.devices[device.device_id]
@@ -138,7 +158,6 @@ class DeviceSession(Device):
         # Filter over device class
         if class_filter is not None:
             for device_id, device in filtered_devices.copy().items():
-                # if not isinstance(device, class_filter):
                 if class_filter in device.qualified_name:
                     del filtered_devices[device_id]
         # Filter over device attributes
@@ -257,9 +276,9 @@ class DeviceSession(Device):
     def start_control_loops(self):
         if not self.get_value(ATTR_LOOPS_RUNNING):
             self.start_asyncore_loop()
+            self.start_device_connection_poller()
             self.start_device_poller_loop()
             self.start_port_read_loop()
-            self.start_client_poller_loop()
             self.set_value(ATTR_LOOPS_RUNNING, True)
 
     def stop_control_loops(self):
@@ -267,7 +286,7 @@ class DeviceSession(Device):
             self.stop_asyncore_loop()
             self.stop_port_read_loop()
             self.stop_device_poller_loop()
-            self.stop_client_poller_loop()
+            self.stop_device_connection_poller()
             self.set_value(ATTR_LOOPS_RUNNING, False)
 
         self.logger.info("Closing all control loops")
@@ -275,13 +294,16 @@ class DeviceSession(Device):
         control_asyncore_loop.join(1)
         control_device_poller.join(1)
         control_port_read_loop.join(1)
-        control_device_client_poller.join(1)
+        control_device_connection_poller.join(1)
+
         self.logger.info("Control loops closed")
 
     def stop(self):
+        self.stop_polling()
+        
         if self.device_server is not None:
             self.device_server.close()
-        self.stop_client_poller_loop()
+
         self.session = None
         del self.devices[self.device_id]
 
@@ -292,14 +314,6 @@ class DeviceSession(Device):
             self.stop_control_loops()
             self.set_value(ATTR_LOOPS_RUNNING, False)
 
-    @staticmethod
-    def start_client_poller_loop():
-        control_device_client_poller.start()
-
-    @staticmethod
-    def stop_client_poller_loop():
-        control_device_client_poller.stop()
-
     @staticmethod
     def start_asyncore_loop():
         control_asyncore_loop.start()
@@ -323,3 +337,29 @@ class DeviceSession(Device):
     @staticmethod
     def stop_port_read_loop():
         control_port_read_loop.stop()
+
+    @staticmethod
+    def start_device_connection_poller():
+        control_device_connection_poller.start()
+
+    @staticmethod
+    def stop_device_connection_poller():
+        control_device_connection_poller.stop()
+
+    def set_high_process_priority(self):
+        """ Set the priority of the process to below-normal."""
+        if psutil.WINDOWS:
+            try:
+                import win32api, win32process, win32con
+            except ImportError:
+                self.logger.error("Package pypiwin32 is not installed")
+            pid = win32api.GetCurrentProcessId()
+            handle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, True, pid)
+            win32process.SetPriorityClass(handle, win32process.REALTIME_PRIORITY_CLASS)
+            # win32process.SetThreadPriority(win32api.GetCurrentThread(), win32process.THREAD_PRIORITY_TIME_CRITICAL)
+        else:
+            try:
+                process = psutil.Process(os.getpid())
+                process.nice(-20)
+            except psutil.AccessDenied as e:
+                self.logger.error("Could not raise process priority")
