@@ -8,7 +8,6 @@ from pint.errors import UndefinedUnitError
 
 from kamzik3 import DeviceUnitError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.devicePort import DevicePort
 from kamzik3.devices.deviceSocket import DeviceSocket
@@ -49,17 +48,16 @@ class DeviceSmaractMcs1(DeviceSocket):
 
     def _init_attributes(self):
         super(DeviceSmaractMcs1, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_CHANNELS: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_HAND_TOOL_MODE: Attribute("Display only", default_type=self.handtool_modes.keys(),
-                                           set_function=self.set_hand_tool_mode),
-            ATTR_SENSOR_MODE: Attribute(None, default_type=self.sensor_modes.keys(), set_function=self.set_sensor_mode,
-                                        set_value_when_set_function=True),
-            ATTR_SENSOR_PRESENT: Attribute(True, readonly=True, default_type=np.bool),
-        })
+        self.create_attribute(ATTR_STATE, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_CHANNELS, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_HAND_TOOL_MODE, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_HAND_TOOL_MODE, default_value="Display only",
+                              default_type=self.handtool_modes.keys(),
+                              set_function=self.set_hand_tool_mode)
+        self.create_attribute(ATTR_SENSOR_MODE, default_type=self.sensor_modes.keys(),
+                              set_function=self.set_sensor_mode)
+        self.create_attribute(ATTR_SENSOR_PRESENT, default_value=True, readonly=True, default_type=np.bool)
 
     def valid_command_format(self, command):
         if command is not None and command[-1] == "\n":
@@ -94,24 +92,24 @@ class DeviceSmaractMcs1(DeviceSocket):
             channel, error_number = [int(v) for v in output[2:].split(",")]
             if error_number > 0:
                 # if error_number == 129:
-                #     self.set_value(ATTR_SENSOR_PRESENT, False)
-                #     self.set_value(ATTR_SENSOR_MODE, "Disabled")
+                #     self.set_raw_value(ATTR_SENSOR_PRESENT, False)
+                #     self.set_raw_value(ATTR_SENSOR_MODE, "Disabled")
                 # for channel in range(self[ATTR_CHANNELS][VALUE]):
                 #     self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
                 #     self.notify((channel, ATTR_SENSOR_PRESENT), False)
                 # elif error_number == 140:
-                #     self.set_value(ATTR_SENSOR_MODE, "Disabled")
+                #     self.set_raw_value(ATTR_SENSOR_MODE, "Disabled")
                 # for channel in range(self[ATTR_CHANNELS][VALUE]):
                 #     self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
                 self.handle_command_error(command, output)
         elif command == ":GSI\n":
-            self.set_value(ATTR_SERIAL_NUMBER, output.strip(":IN"))
+            self.set_raw_value(ATTR_SERIAL_NUMBER, output.strip(":IN"))
         elif command == ":GNC\n":
             channels = int(output.strip(":N"))
-            self.set_value(ATTR_CHANNELS, channels)
+            self.set_raw_value(ATTR_CHANNELS, channels)
         elif command == ":GSE\n":
             mode = self.sensor_modes.inv[int(output.strip(":SE"))]
-            self.set_value(ATTR_SENSOR_MODE, mode)
+            self.set_raw_value(ATTR_SENSOR_MODE, mode)
             for channel in range(self[ATTR_CHANNELS][VALUE]):
                 self.notify((channel, ATTR_SENSOR_MODE), mode)
         elif output[:3] == ":BR":
@@ -193,10 +191,8 @@ class DeviceSmaractMcs1Port(DevicePort, DeviceSmaractMcs1):
 
     def _init_attributes(self):
         super(DeviceSmaractMcs1Port, self)._init_attributes()
-        self.attributes.update({
-            ATTR_BAUDRATE: Attribute(self.baud_rate, readonly=False, default_type=self.BAUDRATES,
-                                     set_function=self.set_baudrate)
-        })
+        self.create_attribute(ATTR_BAUDRATE, default_value=self.baud_rate, readonly=False, default_type=self.BAUDRATES,
+                              set_function=self.set_baudrate)
 
     def handshake(self):
         self.logger.info(u"Handshake initiated")
@@ -218,6 +214,42 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
     sensor_modes = DeviceSmaractMcs1.sensor_modes
     safe_direction_modes = DeviceSmaractMcs1.safe_direction_modes
 
+    def _init_attributes(self):
+        super(DeviceSmaractMcs1Channel, self)._init_attributes()
+        self.create_attribute(ATTR_STATE, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_MOVING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_POSITION, default_type=np.int64, set_function=self._move_absolute,
+                              set_value_when_set_function=False)
+        self.create_attribute(ATTR_REVOLUTIONS, readonly=True, default_type=np.int64)
+        self.create_attribute(ATTR_CALIBRATING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_CALIBRATED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_MOVEMENT_LOCKED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_SENSOR_MODE, readonly=True, default_type=self.sensor_modes.keys())
+        self.create_attribute(ATTR_SENSOR_TYPE, default_type=self.sensor_types, set_function=self.set_sensor_type)
+        self.create_attribute(ATTR_SENSOR_PRESENT, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_ENCODER_BASE_UNIT, readonly=True)
+        self.create_attribute(ATTR_ENCODER_RESOLUTION, readonly=True, default_type=np.int8)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.int64,
+                              set_function=self.set_lower_position_limit)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.int64,
+                              set_function=self.set_upper_position_limit)
+        self.create_attribute(ATTR_STEP_FREQUENCY, default_value=1000, default_type=np.uint32, min_value=1,
+                              max_value=18.5e3, unit="Hz",
+                              set_function=self.set_step_frequency)
+        self.create_attribute(ATTR_STEP_AMPLITUDE, default_value=1000, default_type=np.uint32, min_value=1,
+                              max_value=65535,
+                              set_function=self.set_step_amplitude)
+        self.create_attribute(ATTR_ACCELERATION, default_type=np.int64, min_value=0, max_value=100e9,
+                              set_function=self.set_acceleration)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.int64, min_value=0, max_value=1e8,
+                              set_function=self.set_velocity)
+        self.create_attribute(ATTR_SAFE_DIRECTION, default_type=self.safe_direction_modes.keys(),
+                              set_function=self.set_safe_direction)
+        self.create_attribute(ATTR_HOLDING_TIME, default_value=0, default_type=np.uint16, min_value=0, max_value=60e3,
+                              unit=u"ms")
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -238,42 +270,6 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         self.get_safe_direction()
         self.command(u":GCLS{}\n".format(self.channel), callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DeviceSmaractMcs1Channel, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_MOVING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_POSITION: Attribute(None, default_type=np.int64, set_function=self._move_absolute,
-                                     set_value_when_set_function=False),
-            ATTR_REVOLUTIONS: Attribute(None, readonly=True, default_type=np.int64),
-            ATTR_CALIBRATING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_REFERENCING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_CALIBRATED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_REFERENCED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_MOVEMENT_LOCKED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_SENSOR_MODE: Attribute(None, readonly=True, default_type=self.sensor_modes.keys()),
-            ATTR_SENSOR_TYPE: Attribute(None, default_type=self.sensor_types, set_function=self.set_sensor_type),
-            ATTR_SENSOR_PRESENT: Attribute(True, readonly=True, default_type=np.bool),
-            ATTR_ENCODER_BASE_UNIT: Attribute(None, readonly=True),
-            ATTR_ENCODER_RESOLUTION: Attribute(None, readonly=True, default_type=np.int8),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.int64,
-                                                 set_function=self.set_lower_position_limit),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.int64,
-                                                 set_function=self.set_upper_position_limit),
-            ATTR_STEP_FREQUENCY: Attribute(1000, default_type=np.uint32, min_value=1, max_value=18.5e3, unit="Hz",
-                                           set_function=self.set_step_frequency),
-            ATTR_STEP_AMPLITUDE: Attribute(1000, default_type=np.uint32, min_value=1, max_value=65535,
-                                           set_function=self.set_step_amplitude),
-            ATTR_ACCELERATION: Attribute(None, default_type=np.int64, min_value=0, max_value=100e9,
-                                         set_function=self.set_acceleration),
-            ATTR_VELOCITY: Attribute(None, default_type=np.int64, min_value=0, max_value=1e8,
-                                     set_function=self.set_velocity),
-            ATTR_SAFE_DIRECTION: Attribute(None, default_type=self.safe_direction_modes.keys(),
-                                           set_function=self.set_safe_direction),
-            ATTR_HOLDING_TIME: Attribute(0, default_type=np.uint16, min_value=0, max_value=60e3, unit=u"ms"),
-        })
-
     def subject_update(self, key, value, subject):
         super(DeviceSmaractMcs1Channel, self).subject_update(key, value, subject)
 
@@ -328,8 +324,8 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                 max_position, max_revolutions = value
                 value = max_position + max_revolutions * revolution
                 # self.set_attribute([ATTR_POSITION, MAX], value)
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
     def _set_position_unit(self, sensor_type):
         if sensor_type is None:
@@ -418,7 +414,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move to absolute position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), position)
+            return self.set_value(ATTR_POSITION, position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -428,7 +424,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             self.logger.info(u"Move by relative position {}".format(position))
-            return self.set_attribute((ATTR_POSITION, VALUE), self[ATTR_POSITION].value() + position)
+            return self.set_value(ATTR_POSITION, self[ATTR_POSITION].value() + position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -501,7 +497,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_attribute([ATTR_POSITION, MIN], self[ATTR_POSITION].remove_offset_factor(value))
+        self.set_attribute([ATTR_POSITION, MIN], self[ATTR_POSITION].apply_offset_factor(value))
         self.get_position_limit_range()
         return out
 
@@ -526,7 +522,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_attribute([ATTR_POSITION, MAX], self[ATTR_POSITION].remove_offset_factor(value))
+        self.set_attribute([ATTR_POSITION, MAX], self[ATTR_POSITION].apply_offset_factor(value))
         self.get_position_limit_range()
         return out
 
