@@ -8,7 +8,6 @@ from pint.errors import UndefinedUnitError
 import kamzik3
 from kamzik3 import DeviceError, CommandFormatException, DeviceUnitError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.deviceSocket import DeviceSocket
 from kamzik3.devices.general.deviceScanner import DeviceScanner
@@ -80,17 +79,14 @@ class DeviceSmaractMcs2(DeviceSocket):
 
     def _init_attributes(self):
         super(DeviceSmaractMcs2, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_CHANNELS: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_MODULES: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_HAND_TOOL: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_MOVEMENT_LOCKED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_INTERNAL_COMM_FAILURE: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_STREAMING: Attribute(None, readonly=True, default_type=np.bool),
-        })
+        self.create_attribute(ATTR_STATE, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_CHANNELS, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_MODULES, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_HAND_TOOL, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_MOVEMENT_LOCKED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_INTERNAL_COMM_FAILURE, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_STREAMING, readonly=True, default_type=np.bool)
 
     def command(self, command, callback=None, with_token=False, returning=True):
         if not self.valid_command_format(command):
@@ -167,34 +163,34 @@ class DeviceSmaractMcs2(DeviceSocket):
         command = command.strip()
 
         if command == ":DEV:NOCH?":
-            self.set_attribute((ATTR_CHANNELS, VALUE), int(output))
+            self.set_raw_value(ATTR_CHANNELS, int(output))
         elif command == ":DEV:SNUM?":
-            self.set_attribute((ATTR_SERIAL_NUMBER, VALUE), output.replace('"', ""))
+            self.set_raw_value(ATTR_SERIAL_NUMBER, output.replace('"', ""))
         elif command == ":DEV:NOBM?":
-            self.set_attribute((ATTR_MODULES, VALUE), int(output))
+            self.set_raw_value(ATTR_MODULES, int(output))
             for i in range(int(output)):
                 module_name = "Module{}".format(i)
-                self.attributes[module_name] = {
-                    ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint16),
-                    ATTR_TEMPERATURE: Attribute(readonly=True, default_type=np.float32, decimals=2, unit="degC", ),
-                    ATTR_SENSOR_PRESENT: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_BOOSTER_PRESENT: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_ADJUSTMENT_ACTIVE: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_IOM_PRESENT: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_INTERNAL_COMM_FAILURE: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_HIGH_VOLTAGE_FAILURE: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_HIGH_VOLTAGE_OVERLOAD: Attribute(None, readonly=True, default_type=np.bool),
-                    ATTR_OVER_TEMPERATURE: Attribute(None, readonly=True, default_type=np.bool),
-                }
+                self.create_attribute(ATTR_STATE, module_name, readonly=True, default_type=np.uint16)
+                self.create_attribute(ATTR_TEMPERATURE, module_name, readonly=True, default_type=np.float32, decimals=2,
+                                      unit="degC")
+                self.create_attribute(ATTR_SENSOR_PRESENT, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_BOOSTER_PRESENT, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_ADJUSTMENT_ACTIVE, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_IOM_PRESENT, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_INTERNAL_COMM_FAILURE, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_HIGH_VOLTAGE_FAILURE, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_HIGH_VOLTAGE_OVERLOAD, module_name, readonly=True, default_type=np.bool)
+                self.create_attribute(ATTR_OVER_TEMPERATURE, module_name, readonly=True, default_type=np.bool)
+
                 self.poll_command(":MOD{}:STAT?\r\n".format(i), 2000)
                 self.poll_command(":MOD{}:TEMP?\r\n".format(i), 2000)
         elif command == ":DEV:STAT?":
             output = int(output)
-            self.set_attribute((ATTR_STATE, VALUE), output)
-            self.set_attribute((ATTR_HAND_TOOL, VALUE), bool(output % 0x0001))
-            self.set_attribute((ATTR_MOVEMENT_LOCKED, VALUE), bool(output % 0x0002))
-            self.set_attribute((ATTR_INTERNAL_COMM_FAILURE, VALUE), bool(output % 0x0100))
-            self.set_attribute((ATTR_STREAMING, VALUE), bool(output % 0x1000))
+            self.set_raw_value(ATTR_STATE, output)
+            self.set_raw_value(ATTR_HAND_TOOL, bool(output % 0x0001))
+            self.set_raw_value(ATTR_MOVEMENT_LOCKED, bool(output % 0x0002))
+            self.set_raw_value(ATTR_INTERNAL_COMM_FAILURE, bool(output % 0x0100))
+            self.set_raw_value(ATTR_STREAMING, bool(output % 0x1000))
         elif command == ":SYST:ERR:COUN?":
             if int(output) > 0:
                 self.get_next_error()
@@ -205,18 +201,18 @@ class DeviceSmaractMcs2(DeviceSocket):
             _, module, attribute = command.split(":")
             module_name = "Module{}".format(module[3:])
             if attribute == "TEMP?":
-                self.set_attribute((module_name, ATTR_TEMPERATURE, VALUE), float(output))
+                self.set_raw_value((module_name, ATTR_TEMPERATURE), float(output))
             elif attribute == "STAT?":
                 output = int(output)
-                self.set_attribute((module_name, ATTR_STATE, VALUE), output)
-                self.set_attribute((module_name, ATTR_SENSOR_PRESENT, VALUE), bool(output & 0x0001))
-                self.set_attribute((module_name, ATTR_BOOSTER_PRESENT, VALUE), bool(output & 0x0002))
-                self.set_attribute((module_name, ATTR_ADJUSTMENT_ACTIVE, VALUE), bool(output & 0x0004))
-                self.set_attribute((module_name, ATTR_IOM_PRESENT, VALUE), bool(output & 0x0008))
-                self.set_attribute((module_name, ATTR_INTERNAL_COMM_FAILURE, VALUE), bool(output & 0x0100))
-                self.set_attribute((module_name, ATTR_HIGH_VOLTAGE_FAILURE, VALUE), bool(output & 0x1000))
-                self.set_attribute((module_name, ATTR_HIGH_VOLTAGE_OVERLOAD, VALUE), bool(output & 0x2000))
-                self.set_attribute((module_name, ATTR_OVER_TEMPERATURE, VALUE), bool(output & 0x4000))
+                self.set_raw_value((module_name, ATTR_STATE), output)
+                self.set_raw_value((module_name, ATTR_SENSOR_PRESENT), bool(output & 0x0001))
+                self.set_raw_value((module_name, ATTR_BOOSTER_PRESENT), bool(output & 0x0002))
+                self.set_raw_value((module_name, ATTR_ADJUSTMENT_ACTIVE), bool(output & 0x0004))
+                self.set_raw_value((module_name, ATTR_IOM_PRESENT), bool(output & 0x0008))
+                self.set_raw_value((module_name, ATTR_INTERNAL_COMM_FAILURE), bool(output & 0x0100))
+                self.set_raw_value((module_name, ATTR_HIGH_VOLTAGE_FAILURE), bool(output & 0x1000))
+                self.set_raw_value((module_name, ATTR_HIGH_VOLTAGE_OVERLOAD), bool(output & 0x2000))
+                self.set_raw_value((module_name, ATTR_OVER_TEMPERATURE), bool(output & 0x4000))
         elif command[:5] == ":CHAN":
             chunks = command.split(":")
             channel, attribute = chunks[1], chunks[-1]
@@ -285,6 +281,54 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
     move_modes = DeviceSmaractMcs2.move_modes
     safe_direction_modes = DeviceSmaractMcs2.safe_direction_modes
 
+    def _init_attributes(self):
+        super(DeviceSmaractMcs2Channel, self)._init_attributes()
+        self.create_attribute(ATTR_STATE, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_TEMPERATURE, readonly=True, default_type=np.float16, decimals=2, unit="degC")
+        self.create_attribute(ATTR_MOVING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_POSITION, readonly=False, default_type=np.int64, set_function=self._move_absolute,
+                              set_value_when_set_function=False)
+        self.create_attribute(ATTR_TARGET_POSITION, default_type=np.int64, set_function=self._move_absolute,
+                              set_value_when_set_function=False)
+        self.create_attribute(ATTR_CLOSED_LOOP_ACTIVE, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_CALIBRATING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_MOVE_DELAYED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_SENSOR_PRESENT, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_CALIBRATED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_END_STOP_REACHED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_RANGE_LIMIT_REACHED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_FOLLOWING_LIMIT_REACHED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_MOVEMENT_FAILED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_STREAMING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_OVER_TEMPERATURE, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_INDEX_MARK, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_ENCODER_BASE_UNIT, readonly=True)
+        self.create_attribute(ATTR_ENCODER_RESOLUTION, readonly=True, default_type=np.int8)
+        self.create_attribute(ATTR_POSITIONER_CODE, readonly=False, default_type=np.int8, min_value=300, max_value=395,
+                              set_function=self.set_positioner_code)
+        self.create_attribute(ATTR_POSITIONER_TYPE, readonly=True)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.int64,
+                              set_function=self.set_lower_position_limit)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.int64,
+                              set_function=self.set_upper_position_limit)
+        self.create_attribute(ATTR_STEP_FREQUENCY, default_type=np.uint32, min_value=1, max_value=20e3, unit="Hz",
+                              set_function=self.set_step_frequency)
+        self.create_attribute(ATTR_STEP_AMPLITUDE, default_type=np.uint32, min_value=1, max_value=65535,
+                              set_function=self.set_step_amplitude)
+        self.create_attribute(ATTR_ACCELERATION, default_type=np.int64, min_value=0, max_value=100e9,
+                              set_function=self.set_acceleration)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.int64, min_value=0, max_value=10e12,
+                              set_function=self.set_velocity)
+        self.create_attribute(ATTR_SENSOR_MODE, default_type=self.sensor_modes.keys(),
+                              set_function=self.set_sensor_mode)
+        self.create_attribute(ATTR_MOVE_MODE, default_type=self.move_modes.keys(), set_function=self.set_move_mode)
+        self.create_attribute(ATTR_SAFE_DIRECTION, default_type=self.safe_direction_modes.keys(),
+                              set_function=self.set_safe_direction)
+        self.create_attribute(ATTR_HOLDING_TIME, default_type=np.int32, min_value=-1, unit="ms",
+                              set_function=self.set_holding_time)
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -312,56 +356,6 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
         self.get_holding_time()
         self.command(u":CHAN{}:SDIR?\r\n".format(self.channel), callback=_finish_configuration)
 
-    def _init_attributes(self):
-        super(DeviceSmaractMcs2Channel, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_TEMPERATURE: Attribute(readonly=True, default_type=np.float16, decimals=2, unit="degC"),
-            ATTR_MOVING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_POSITION: Attribute(None, readonly=False, default_type=np.int64, set_function=self._move_absolute,
-                                     set_value_when_set_function=False),
-            ATTR_TARGET_POSITION: Attribute(None, default_type=np.int64, set_function=self._move_absolute,
-                                            set_value_when_set_function=False),
-            ATTR_CLOSED_LOOP_ACTIVE: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_CALIBRATING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_REFERENCING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_MOVE_DELAYED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_SENSOR_PRESENT: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_CALIBRATED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_REFERENCED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_END_STOP_REACHED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_RANGE_LIMIT_REACHED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_FOLLOWING_LIMIT_REACHED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_MOVEMENT_FAILED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_STREAMING: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_OVER_TEMPERATURE: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_INDEX_MARK: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_ENCODER_BASE_UNIT: Attribute(None, readonly=True),
-            ATTR_ENCODER_RESOLUTION: Attribute(None, readonly=True, default_type=np.int8),
-            ATTR_POSITIONER_CODE: Attribute(None, readonly=False, default_type=np.int8, min_value=300, max_value=395,
-                                            set_function=self.set_positioner_code),
-            ATTR_POSITIONER_TYPE: Attribute(None, readonly=True),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.int64,
-                                                 set_function=self.set_lower_position_limit),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.int64,
-                                                 set_function=self.set_upper_position_limit),
-            ATTR_STEP_FREQUENCY: Attribute(None, default_type=np.uint32, min_value=1, max_value=20e3, unit="Hz",
-                                           set_function=self.set_step_frequency),
-            ATTR_STEP_AMPLITUDE: Attribute(None, default_type=np.uint32, min_value=1, max_value=65535,
-                                           set_function=self.set_step_amplitude),
-            ATTR_ACCELERATION: Attribute(None, default_type=np.int64, min_value=0, max_value=100e9,
-                                         set_function=self.set_acceleration),
-            ATTR_VELOCITY: Attribute(None, default_type=np.int64, min_value=0, max_value=10e12,
-                                     set_function=self.set_velocity),
-            ATTR_SENSOR_MODE: Attribute(None, default_type=self.sensor_modes.keys(), set_function=self.set_sensor_mode),
-            ATTR_MOVE_MODE: Attribute(None, default_type=self.move_modes.keys(), set_function=self.set_move_mode),
-            ATTR_SAFE_DIRECTION: Attribute(None, default_type=self.safe_direction_modes.keys(),
-                                           set_function=self.set_safe_direction),
-            ATTR_HOLDING_TIME: Attribute(None, default_type=np.int32, min_value=-1, unit="ms",
-                                         set_function=self.set_holding_time),
-        })
-
     def start_polling(self):
         super(DeviceSmaractMcs2Channel, self).start_polling()
         self.poll_command(":CHAN{}:STAT?\r\n".format(self.channel), 250)
@@ -382,21 +376,21 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
             attribute = key[1]
 
             if attribute == ATTR_STATE and self.get_value(ATTR_STATE) != value:
-                self.set_attribute((ATTR_MOVING, VALUE), bool(value & 0x0001))
-                self.set_attribute((ATTR_CLOSED_LOOP_ACTIVE, VALUE), bool(value & 0x0002))
-                self.set_attribute((ATTR_CALIBRATING, VALUE), bool(value & 0x0004))
-                self.set_attribute((ATTR_REFERENCING, VALUE), bool(value & 0x0008))
-                self.set_attribute((ATTR_MOVE_DELAYED, VALUE), bool(value & 0x0100))
-                self.set_attribute((ATTR_SENSOR_PRESENT, VALUE), bool(value & 0x0020))
-                self.set_attribute((ATTR_CALIBRATED, VALUE), bool(value & 0x0040))
-                self.set_attribute((ATTR_REFERENCED, VALUE), bool(value & 0x0080))
-                self.set_attribute((ATTR_END_STOP_REACHED, VALUE), bool(value & 0x0100))
-                self.set_attribute((ATTR_RANGE_LIMIT_REACHED, VALUE), bool(value & 0x0200))
-                self.set_attribute((ATTR_FOLLOWING_LIMIT_REACHED, VALUE), bool(value & 0x0400))
-                self.set_attribute((ATTR_MOVEMENT_FAILED, VALUE), bool(value & 0x0800))
-                self.set_attribute((ATTR_STREAMING, VALUE), bool(value & 0x1000))
-                self.set_attribute((ATTR_OVER_TEMPERATURE, VALUE), bool(value & 0x4000))
-                self.set_attribute((ATTR_INDEX_MARK, VALUE), bool(value & 0x8000))
+                self.set_raw_value(ATTR_MOVING, bool(value & 0x0001))
+                self.set_raw_value(ATTR_CLOSED_LOOP_ACTIVE, bool(value & 0x0002))
+                self.set_raw_value(ATTR_CALIBRATING, bool(value & 0x0004))
+                self.set_raw_value(ATTR_REFERENCING, bool(value & 0x0008))
+                self.set_raw_value(ATTR_MOVE_DELAYED, bool(value & 0x0100))
+                self.set_raw_value(ATTR_SENSOR_PRESENT, bool(value & 0x0020))
+                self.set_raw_value(ATTR_CALIBRATED, bool(value & 0x0040))
+                self.set_raw_value(ATTR_REFERENCED, bool(value & 0x0080))
+                self.set_raw_value(ATTR_END_STOP_REACHED, bool(value & 0x0100))
+                self.set_raw_value(ATTR_RANGE_LIMIT_REACHED, bool(value & 0x0200))
+                self.set_raw_value(ATTR_FOLLOWING_LIMIT_REACHED, bool(value & 0x0400))
+                self.set_raw_value(ATTR_MOVEMENT_FAILED, bool(value & 0x0800))
+                self.set_raw_value(ATTR_STREAMING, bool(value & 0x1000))
+                self.set_raw_value(ATTR_OVER_TEMPERATURE, bool(value & 0x4000))
+                self.set_raw_value(ATTR_INDEX_MARK, bool(value & 0x8000))
 
                 if bool(value & 0x0001):
                     self.set_status(STATUS_BUSY)
@@ -421,7 +415,8 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                 self.set_attribute([ATTR_POSITION, MIN], value)
             elif attribute == ATTR_POSITION_UPPER_LIMIT:
                 self.set_attribute([ATTR_POSITION, MAX], value)
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
+
+            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
             self.set_value(attribute, value)
 
     @expose_method()
@@ -449,7 +444,7 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                 position = position.m - self[ATTR_POSITION][VALUE]
             else:
                 position = position.m
-            return self.set_attribute((ATTR_TARGET_POSITION, VALUE), position)
+            return self.set_value(ATTR_TARGET_POSITION, position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -464,7 +459,7 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                 position = self[ATTR_POSITION][VALUE] + position
             else:
                 position = position.m
-            return self.set_attribute((ATTR_TARGET_POSITION, VALUE), position)
+            return self.set_value(ATTR_TARGET_POSITION, position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -573,9 +568,7 @@ class DeviceMCS2FlyScanner(DeviceScanner):
     def _init_attributes(self):
         DeviceScanner._init_attributes(self)
         last_scan = kamzik3.session.get_scan_count()
-        self.attributes.update({
-            ATTR_DWELL_TIME: Attribute(100, default_type=np.float32, min_value=1, unit="ms"),
-        })
+        self.create_attribute(ATTR_DWELL_TIME, default_value=100, default_type=np.float32, min_value=1, unit="ms")
 
     @expose_method()
     def get_scanner_attributes(self):
