@@ -6,7 +6,6 @@ from pint import UndefinedUnitError
 
 from kamzik3 import units, DeviceUnitError
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.deviceChannel import DeviceChannel
 from kamzik3.devices.deviceSocket import DeviceSocket
 from kamzik3.snippets.snippetsDecorators import expose_method
@@ -78,38 +77,35 @@ class DeviceSmarpodEcm(DeviceSocket):
 
     def __init__(self, host, port=2000, device_id=None, config=None):
         super().__init__(host, port, device_id, config)
-        self._expose_methods_to_clients()
 
     def _init_attributes(self):
         super(DeviceSmarpodEcm, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_SERIAL_NUMBER: Attribute(readonly=True),
-            ATTR_CHANNELS: Attribute(6, readonly=True, default_type=np.uint16),
-            ATTR_ACTIVE_UNIT: Attribute(None, readonly=True, default_type=np.uint16),
-            ATTR_STEP_FREQUENCY: Attribute(None, default_type=np.uint32, min_value=1, max_value=20e3, unit="Hz",
-                                           set_function=self.set_step_frequency),
-            ATTR_REFERENCED: Attribute(None, readonly=True, default_type=np.bool),
-            ATTR_SENSOR_MODE: Attribute(None, default_type=self.sensor_modes.keys(), set_function=self.set_sensor_mode),
-            ATTR_REFERENCING_METHOD: Attribute(None, default_type=self.referencing_method,
-                                               set_function=self.set_referencing_method),
-            ATTR_REFERENCING_DIRECTION_X: Attribute(None, default_type=self.referencing_direction,
-                                                    set_function=self.set_referencing_direction_x),
-            ATTR_REFERENCING_DIRECTION_Y: Attribute(None, default_type=self.referencing_direction,
-                                                    set_function=self.set_referencing_direction_y),
-            ATTR_REFERENCING_DIRECTION_Z: Attribute(None, default_type=self.referencing_direction,
-                                                    set_function=self.set_referencing_direction_z),
-            ATTR_PIVOT_MODE: Attribute(None, default_type=self.pivot_mode,
-                                       set_function=self.set_pivot_mode),
-            ATTR_VELOCITY: Attribute(None, default_type=np.float64, unit="m/s", set_function=self.set_velocity,
-                                     decimals=9),
-            ATTR_PIVOT_X: Attribute(None, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_x),
-            ATTR_PIVOT_Y: Attribute(None, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_y),
-            ATTR_PIVOT_Z: Attribute(None, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_z),
-            ATTR_POSITIONS: Attribute(None, readonly=True),
-            ATTR_CALIBRATING: Attribute(False, readonly=True, default_type=np.bool),
-            ATTR_REFERENCING: Attribute(False, readonly=True, default_type=np.bool),
-        })
+        self.create_attribute(ATTR_SERIAL_NUMBER, readonly=True)
+        self.create_attribute(ATTR_CHANNELS, default_value=6, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_ACTIVE_UNIT, readonly=True, default_type=np.uint16)
+        self.create_attribute(ATTR_STEP_FREQUENCY, default_type=np.uint32, min_value=1, max_value=20e3, unit="Hz",
+                              set_function=self.set_step_frequency)
+        self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_SENSOR_MODE, default_type=self.sensor_modes.keys(),
+                              set_function=self.set_sensor_mode)
+        self.create_attribute(ATTR_REFERENCING_METHOD, default_type=self.referencing_method,
+                              set_function=self.set_referencing_method)
+        self.create_attribute(ATTR_REFERENCING_DIRECTION_X, default_type=self.referencing_direction,
+                              set_function=self.set_referencing_direction_x)
+        self.create_attribute(ATTR_REFERENCING_DIRECTION_Y, default_type=self.referencing_direction,
+                              set_function=self.set_referencing_direction_y)
+        self.create_attribute(ATTR_REFERENCING_DIRECTION_Z, default_type=self.referencing_direction,
+                              set_function=self.set_referencing_direction_z)
+        self.create_attribute(ATTR_PIVOT_MODE, default_type=self.pivot_mode,
+                              set_function=self.set_pivot_mode)
+        self.create_attribute(ATTR_VELOCITY, default_type=np.float64, unit="m/s", set_function=self.set_velocity,
+                              decimals=9)
+        self.create_attribute(ATTR_PIVOT_X, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_x)
+        self.create_attribute(ATTR_PIVOT_Y, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_y)
+        self.create_attribute(ATTR_PIVOT_Z, default_type=np.int64, unit=u"nm", set_function=self.set_pivot_z)
+        self.create_attribute(ATTR_POSITIONS, readonly=True)
+        self.create_attribute(ATTR_CALIBRATING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCING, readonly=True, default_type=np.bool)
 
     def valid_command_format(self, command):
         if command is not None and command.endswith(u"\r\n"):
@@ -170,37 +166,37 @@ class DeviceSmarpodEcm(DeviceSocket):
             for channel in range(self.get_value(ATTR_CHANNELS)):
                 self.notify((channel, ATTR_STATUS), motor_status)
         elif command == u"%unit?\r\n":
-            self.set_value(ATTR_ACTIVE_UNIT, int(output))
+            self.set_raw_value(ATTR_ACTIVE_UNIT, int(output))
         elif command == u"sen?\r\n":
-            self.set_value(ATTR_SENSOR_MODE, self.sensor_modes.inv[int(output)])
+            self.set_raw_value(ATTR_SENSOR_MODE, self.sensor_modes.inv[int(output)])
         elif command == u"piv?\r\n":
             pivot_x, pivot_y, pivot_z = output.split(" ")
             pivot_x = device_units(self, ATTR_PIVOT_X, units.Quantity(float(pivot_x), "m")).m
             pivot_y = device_units(self, ATTR_PIVOT_Y, units.Quantity(float(pivot_y), "m")).m
             pivot_z = device_units(self, ATTR_PIVOT_Z, units.Quantity(float(pivot_z), "m")).m
-            self.set_value(ATTR_PIVOT_X, pivot_x)
-            self.set_value(ATTR_PIVOT_Y, pivot_y)
-            self.set_value(ATTR_PIVOT_Z, pivot_z)
+            self.set_raw_value(ATTR_PIVOT_X, pivot_x)
+            self.set_raw_value(ATTR_PIVOT_Y, pivot_y)
+            self.set_raw_value(ATTR_PIVOT_Z, pivot_z)
         elif command == u"pos?\r\n":
-            self.set_value(ATTR_POSITIONS, output)
+            self.set_raw_value(ATTR_POSITIONS, output)
             for channel, position in enumerate(map(float, output.split(" "))):
                 self.notify((channel, ATTR_POSITION), position)
         elif command == u"ref?\r\n":
-            self.set_value(ATTR_REFERENCED, bool(int(output)))
+            self.set_raw_value(ATTR_REFERENCED, bool(int(output)))
         elif command == u"vel?\r\n":
-            self.set_value(ATTR_VELOCITY, float(output))
+            self.set_raw_value(ATTR_VELOCITY, float(output))
         elif command == u"get fref-method\r\n":
-            self.set_value(ATTR_REFERENCING_METHOD, output)
+            self.set_raw_value(ATTR_REFERENCING_METHOD, output)
         elif command == u"get fref-x-direction\r\n":
-            self.set_value(ATTR_REFERENCING_DIRECTION_X, output)
+            self.set_raw_value(ATTR_REFERENCING_DIRECTION_X, output)
         elif command == u"get fref-y-direction\r\n":
-            self.set_value(ATTR_REFERENCING_DIRECTION_Y, output)
+            self.set_raw_value(ATTR_REFERENCING_DIRECTION_Y, output)
         elif command == u"get fref-z-direction\r\n":
-            self.set_value(ATTR_REFERENCING_DIRECTION_Z, output)
+            self.set_raw_value(ATTR_REFERENCING_DIRECTION_Z, output)
         elif command == u"get pivot-mode\r\n":
-            self.set_value(ATTR_PIVOT_MODE, output)
+            self.set_raw_value(ATTR_PIVOT_MODE, output)
         elif command == u"frq?\r\n":
-            self.set_value(ATTR_STEP_FREQUENCY, int(float(output)))
+            self.set_raw_value(ATTR_STEP_FREQUENCY, int(float(output)))
 
         if callback is not None:
             self.handle_readout_callback(callback, command, readout_buffer)
@@ -340,6 +336,22 @@ class DeviceSmarpodEcmChannel(DeviceChannel):
         self.base_unit = u"m"
         super().__init__(device, channel, device_id, config)
 
+    def _init_attributes(self):
+        super(DeviceSmarpodEcmChannel, self)._init_attributes()
+        if self.channel > 2:
+            self.default_unit = "ndeg"
+            self.base_unit = u"deg"
+
+        self.create_attribute(ATTR_POSITION, default_type=np.float64, decimals=9,
+                              set_function=self._move_absolute, set_value_when_set_function=False,
+                              unit=self.default_unit)
+        self.create_attribute(ATTR_POSITION_UPPER_LIMIT, default_type=np.float64, decimals=9,
+                              set_function=self.set_position_upper_limit, unit=self.default_unit)
+        self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.float64, decimals=9,
+                              set_function=self.set_position_lower_limit, unit=self.default_unit)
+        self.create_attribute(ATTR_CALIBRATING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_REFERENCING, readonly=True, default_type=np.bool)
+
     def handle_configuration(self):
         if self.configured:
             return
@@ -355,24 +367,6 @@ class DeviceSmarpodEcmChannel(DeviceChannel):
         self.connected = True
         _finish_configuration()
 
-    def _init_attributes(self):
-        super(DeviceSmarpodEcmChannel, self)._init_attributes()
-        if self.channel > 2:
-            self.default_unit = "ndeg"
-            self.base_unit = u"deg"
-
-        self.attributes.update({
-            ATTR_POSITION: Attribute(None, default_type=np.float64, decimals=9,
-                                     set_function=self._move_absolute, set_value_when_set_function=False,
-                                     unit=self.default_unit),
-            ATTR_POSITION_UPPER_LIMIT: Attribute(None, default_type=np.float64, decimals=9,
-                                                 set_function=self.set_position_upper_limit, unit=self.default_unit),
-            ATTR_POSITION_LOWER_LIMIT: Attribute(None, default_type=np.float64, decimals=9,
-                                                 set_function=self.set_position_lower_limit, unit=self.default_unit),
-            ATTR_CALIBRATING: Attribute(False, readonly=True, default_type=np.bool),
-            ATTR_REFERENCING: Attribute(False, readonly=True, default_type=np.bool),
-        })
-
     def subject_update(self, key, value, subject):
         super(DeviceSmarpodEcmChannel, self).subject_update(key, value, subject)
 
@@ -382,8 +376,8 @@ class DeviceSmarpodEcmChannel(DeviceChannel):
             if attribute == ATTR_POSITION:
                 value = device_units(self, ATTR_POSITION, units.Quantity(float(value), self.base_unit)).m
 
-            self.set_attribute((ATTR_LATENCY, VALUE), self.device.get_value(ATTR_LATENCY))
-            self.set_value(attribute, value)
+            self.set_raw_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
+            self.set_raw_value(attribute, value)
 
     def _move_absolute(self, position, callback=None):
         position = device_units(self, ATTR_POSITION, position).to(self.base_unit)
@@ -395,7 +389,7 @@ class DeviceSmarpodEcmChannel(DeviceChannel):
         try:
             position = device_units(self, ATTR_POSITION, position)
             current_position = self[ATTR_POSITION].value()
-            return self.set_attribute([ATTR_POSITION, VALUE], current_position + position, callback)
+            return self.set_value(ATTR_POSITION, current_position + position, callback)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
@@ -404,7 +398,7 @@ class DeviceSmarpodEcmChannel(DeviceChannel):
     def move_absolute(self, position, callback=None):
         try:
             position = device_units(self, ATTR_POSITION, position)
-            return self.set_attribute((ATTR_POSITION, VALUE), position.m, callback)
+            return self.set_value(ATTR_POSITION, position.m, callback)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
             raise DeviceUnitError(e)
