@@ -20,13 +20,11 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
     def __init__(self, device=None, config=None, parent=None):
         self.groups = []
         DeviceWidget.__init__(self, device, config=config, parent=parent)
-        self.slot_set_status(self.device.get_value(ATTR_STATUS))
 
     def setupUi(self, form):
         super(DeviceDebugWidget, self).setupUi(form)
         self.button_connect.hide()
         self.button_enable.hide()
-        self.sigNewCommand.connect(self.output_commands.setPlainText)
         self.label_device_id.setText(self.device_id)
         self.button_enable.setEnabled(False)
         self.button_disable.setEnabled(False)
@@ -35,6 +33,10 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
         self.setWindowTitle(u"{} debug".format(self.device_id))
         self.setWindowIcon(QIcon(":/icons/icons/settings.png"))
 
+    def init_signals(self):
+        super().init_signals()
+        self.sigNewCommand.connect(self.output_commands.setPlainText)
+
     @pyqtSlot()
     def slot_handle_configuration(self):
         DeviceWidget.slot_handle_configuration(self)
@@ -187,7 +189,7 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
                             inputs_layout.addWidget(input_value)
                             argument_inputs.append(input_value)
                         else:
-                            attribute_widget = AttributeDisplayWidget(attribute_title, device_attribute)
+                            attribute_widget = AttributeDisplayWidget(attribute_title, device_attribute.attribute_copy())
                             attribute_widget.name = attribute_title
                             inputs_layout.addWidget(attribute_widget.label_widget)
                             inputs_layout.addWidget(attribute_widget.input_widget)
@@ -231,7 +233,6 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
     def _generate_attribute_groups(self, attributes, group, attribute_path):
         column, row = 0, 0
         columns = 4 if self.config is None else self.config.get("columns", 4)
-
         for key, value in attributes.items():
             if Attribute.is_attribute(value):
                 if column % columns == 0:
