@@ -24,9 +24,13 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
     def __init__(self, title=None, config=None, parent=None):
         MacroWidget.__init__(self, parent)
         self.config = config
+        self.title = title
         if self.config is None:
             self.config = {}
         self.setupUi(self)
+
+    def setupUi(self, Form):
+        super().setupUi(Form)
         self.button_settings.setEnabled(False)
 
         if self.config.get("input", "relative") == "relative":
@@ -68,7 +72,7 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
         self.button_group.addButton(self.field_size_lock_button)
 
         self.spacer = self.gridLayout.itemAtPosition(5, 0)
-        self.set_title(title)
+        self.set_title(self.title)
         self._init_device_list()
         self._init_scanner_list()
 
@@ -203,12 +207,15 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
 
             if self.config.get("scanner", "None") != "None":
                 scanner = kamzik3.session.get_device(self.config["scanner"])
-                scanner_attributes = scanner.get_scanner_attributes()
-                for attribute in scanner_attributes:
-                    self.macro_widgets.append(
-                        self._show_method_attribute_widget(attribute, scanner.get_attribute(attribute).attribute_copy(),
-                                                           row))
-                    row += 1
+                try:
+                    scanner_attributes = scanner.get_scanner_attributes()
+                    for attribute in scanner_attributes:
+                        self.macro_widgets.append(
+                            self._show_method_attribute_widget(attribute, scanner.get_attribute(attribute).attribute_copy(),
+                                                               row))
+                        row += 1
+                except AttributeError:
+                    pass
 
             self.layout().removeItem(self.spacer)
             self.layout().addItem(self.spacer, row, 0)
@@ -311,10 +318,6 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
                 units.Quantity(math.ceil(field_size / values["step_size"]))
             )
 
-    """
-    Signals and checks for relative scan
-    """
-
     def signal_field_size_changed(self, __):
         values = self.get_input_values()
         self.macro_inputs["step_size"].input_widget.blockSignals(True)
